# JavaScript interoperability
The `nodejs_interop.js` library provides interoperability with the [JavaScript](https://developer.mozilla.org/en-US/docs/Web/JavaScript) APIs, and helper functions for converting or inspecting objects.

```dart
import 'package:nodejs_interop/js.dart';
```

## Classes
The `nodejs_interop.js` library provides bindings for the following JavaScript classes:

### `JSError`
An `Exception` that wraps the JavaScript [`Error`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Error) class:

```dart
import 'package:nodejs_interop/node.dart' show require;

try { require('./not_found.js'); }
on JSError { print('Unable to require the "not_found" module'); }
```

The library also provides bindings for the child classes of the `Error` class:

- `JSEvalError`
- `JSRangeError`
- `JSReferenceError`
- `JSSyntaxError`
- `JSTypeError`
- `JSUriError`

### `Promise<T>`
A generic that wraps the JavaScript [`Promise`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Promise) class:

```dart
import 'dart:js' show allowInterop;

// Resolved promises.
var promise1 = Promise.resolve<String>('foo');
var promise2 = new Promise<String>(allowInterop((resolve, reject) => resolve('bar')));

// Rejected promises.
var error = new JSError('Promise rejected');
var promise3 = Promise.reject(error);
var promise4 = new Promise(allowInterop((resolve, reject) => reject(error)));
```

## Functions
The `nodejs_interop.js` library exports these functions from the `dart:js_util` library:

- `callConstructor()`
- `callMethod()`
- `getProperty()`
- `hasProperty()`
- `instanceof()`
- `jsify()`
- `newObject()`
- `setProperty()`

It also provides the following helper functions:

### `Future<T> asFuture<T>(Promise<T> promise)`
Converts a JavaScript `Promise` to a Dart `Future`:

```dart
var future = asFuture<String>(Promise.resolve('foo'));
print(await future); // "foo"
```

### `Promise<T> asPromise<T>(Future<T> future)`
Converts a Dart `Future` to a JavaScript `Promise`:

```dart
import 'dart:async' show Future;
import 'dart:js' show allowInterop;

var promise = asPromise<String>(new Future.value('foo'));
promise.then(allowInterop(print)); // "foo"
```

### `dynamic dartify(Object value)`
Recursively converts a JavaScript value to a Dart object:

```dart
var jsArray = jsify(['foo', 'bar', {'baz': 'qux'}]);
var dartList = dartify(jsArray);

print(dartList is List); // true
print(dartList.first is String); // true
print(dartList.last is Map); // true

var jsObject = jsify({'foo': 'bar', 'baz': ['qux']});
var dartMap = dartify(jsObject);

print(dartMap is Map); // true
print(dartMap.values.first is String); // true
print(dartMap.values.last is List); // true
```

### `bool deleteProperty(Object object, Object propertyKey)`
Removes a property from a JavaScript object:

```dart
var jsObject = jsify({'foo': 'bar'});
print(hasProperty(jsObject, 'foo')); // true

deleteProperty(jsObject, 'foo');
print(hasProperty(jsObject, 'foo')); // false
```

### `Map getProperties(Object object)`
Returns a map of the own enumerable properties of a JavaScript object:

```dart
var jsObject = jsify({'foo': 'bar'});
var properties = getProperties(jsObject);
print(properties.keys.single); // "foo"
print(properties['foo']); // "bar"
```
