# Node.js interoperability
The `nodejs_interop.node` library provides interoperability with the [Node.js](https://nodejs.org) core modules.

```dart
import 'package:nodejs_interop/node.dart';
```

## Functions
The `nodejs_interop.node` library provides the following helper functions:

### `Stream<T> asStream<T>(Readable<T> readable)`
Converts a Node.js `Readable` stream to a Dart `Stream`:

```dart
// TODO Provide a code sample.
```

### `Future<Null> fn1cb0ToFuture(void fn(arg1, CompletionCallback0 callback), arg1)`
Converts a function following the Node.js callback style to a `Future`:

```dart
// TODO Provide a code sample.
```

This function is available in several variants based on the arity of the callback-based function:

- `fn1cb1ToFuture()`
- `fn1cb2ToFuture()`
- `fn2cb0ToFuture()`
- `fn2cb1ToFuture()`
- `fn2cb2ToFuture()`
- `fn3cb0ToFuture()`
- etc.

### `T loadLibrary<T>(String identifier, {T onError(String identifier)})`
Requires a Node.js module:

```dart
final os = loadLibrary<OSModule>('os');
print(os.arch()); // "x64"
```

This function is prefered to the Node.js `require()` function because it properly exposes to the Dart engine the classes exported by the required module:

```dart
// Instead of importing a module by using the `require()` function...
UrlModule url = require('url');
var urlObject = new Url('https://nodejs.org');
  // => Error: the `url.URL` native class is not exposed.

// ...use the `loadLibrary()` function to import the exported classes.
final url = loadLibrary<UrlModule>('url');
var urlObject = new Url('https://www.dartlang.org');
  // => OK: the `url.URL` native class is properly exposed.
```

TODO Document the `onError` argument.

## Core modules
The `nodejs_interop.js` library provides bindings for the following Node.js modules:

### The [globals](https://nodejs.org/api/globals.html) module
This module provides objects that are available globally:

```dart
print(dirname); // "/path/to"
print(filename); // "app.dart.js"
print(module.id); // "/path/to/app.dart.js"
print(process.arch); // "x64"
print(process.platform); // "linux"
```

### The [`assert`](https://nodejs.org/api/assert.html) module
This module provides a simple set of assertion tests that can be used to test invariants:

```dart
AssertModule _assert = require('assert');
_assert(true); // Returns normally.
_assert(false); // Throws a `JSAssertionError`.
```

### The [`buffer`](https://nodejs.org/api/buffer.html) module
This module provides the global `Buffer` class and related utilities:

```dart
import 'dart:convert' show BASE64;
import 'dart:js_util' show callMethod;

var buffer = Buffer.from('hello world', BufferEncoding.ascii);
print(buffer.toString()); // "hello world"
print(BASE64.encode(buffer)); // "aGVsbG8gd29ybGQ="
print(callMethod(buffer, 'toString', [BufferEncoding.base64])); // "aGVsbG8gd29ybGQ="
print(callMethod(buffer, 'toString', [BufferEncoding.hex])); // "68656c6c6f20776f726c64"
```

> Instances of the `Buffer` class are casted to `Uint8List` by the Dart engine.
> The methods and properties specific to the `Buffer` class are still accessible
> using the `callMethod()` and `getProperty()` functions from the `dart:js_util`
> or `nodejs_interop.js` libraries.

### The [`child_process`](https://nodejs.org/api/child_process.html) module
This module provides the ability to spawn child processes:

```dart
import 'dart:js' show allowInterop;

ChildProcessModule child_process = require('child_process');
var callback = allowInterop((error, stdout, stderr) => print(stdout));
child_process.exec('cat *.dart | wc -l', callback);
```

### The [`cluster`](https://nodejs.org/api/cluster.html) module
This module allows easy creation of child processes that all share server ports:

```dart
ClusterModule cluster = require('cluster');
TODO: code sample!
```

### The [`console`](https://nodejs.org/api/console.html) module
This module provides a simple debugging console that is similar to the JavaScript console mechanism provided by web browsers:

```dart
// Required: the `Console` native class doesn't exist before this call.
loadLibrary('console');
TODO: code sample!
```

### The [`dns`](https://nodejs.org/api/dns.html) module
This module provides methods to perform domain name resolution:

```dart
DnsModule dns = require('dns');
TODO: code sample!
```

### The [`events`](https://nodejs.org/api/events.html) module
This module provides the `EventEmitter` class:

```dart
// Required: the `EventEmitter` native class doesn't exist before this call.
loadLibrary('events');

var listener = allowInterop((data) => print('A "eventName" event occurred: ${data['foo']}'));
var emitter = new EventEmitter()..on('eventName', listener);
emitter.emit('eventName', jsify({'foo': 'bar'}));
```

### The [`fs`](https://nodejs.org/api/fs.html) module
This module provides file I/O based on standard POSIX functions:

```dart
FSModule fs = require('fs');
TODO: code sample!
```

### The [`os`](https://nodejs.org/api/os.html) module
This module provides a number of operating system-related utility methods:

```dart
OSModule os = require('os');
print(os.homedir()); // "/home/user"
print(os.tmpdir()); // "/tmp"
```

### The [`path`](https://nodejs.org/api/path.html) module
This module provides utilities for working with file and directory paths:

```dart
PathModule path = require('path');
print(path.basename('/foo/bar/baz.qux')); // "baz.qux"
print(path.dirname('/foo/bar/baz.qux')); // "/foo/bar"
```

### The [`querystring`](https://nodejs.org/api/querystring.html) module
This module provides utilities for parsing and formatting URL query strings:

```dart
QueryStringModule querystring = require('querystring');
TODO: code sample!
```
