part of nodejs_interop.io;

/// The mode for opening a file for reading and writing to the end of it.
/// The file is created if it does not already exist.
const f.FileMode APPEND = f.FileMode.APPEND; // ignore: constant_identifier_names

/// The mode for opening a file only for reading.
const f.FileMode READ = f.FileMode.READ; // ignore: constant_identifier_names

/// The mode for opening a file for reading and writing.
/// The file is overwritten if it already exists. The file is created if it does not already exist.
const f.FileMode WRITE = f.FileMode.WRITE; // ignore: constant_identifier_names

/// Mode for opening a file for writing only.
/// The file is overwritten if it already exists. The file is created if it does not already exist.
const f.FileMode WRITE_ONLY = f.FileMode.WRITE_ONLY; // ignore: constant_identifier_names

/// Mode for opening a file for writing only to the end of it.
/// The file is created if it does not already exist.
const f.FileMode WRITE_ONLY_APPEND = f.FileMode.WRITE_ONLY_APPEND; // ignore: constant_identifier_names

/// A reference to a file on the file system.
class File extends FileSystemEntity implements f.File {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// Creates a new file.
  File(String path): super._internal(path);

  /// Creates a new file from the specified [uri].
  File.fromUri(Uri uri): this(uri.toFilePath(windows: const Platform().isWindows));

  /// A [File] whose path is the absolute path to this entity.
  @override
  File get absolute => new File(fileSystem.path.absolute(path));

  /// A [Uri] representing the entity's location.
  @override
  Uri get uri => new Uri.file(path, windows: const Platform().isWindows);

  /// Copy this file. Returns a [Future] that completes with a [File] instance for the copied file.
  ///
  /// If [newPath] identifies an existing file, that file is replaced.
  /// If [newPath] identifies an existing directory the operation fails and the future completes with an exception.
  @override
  Future<File> copy(String newPath) async {
    try {
      await fn2cb0ToFuture(_fs.copyFile, path, newPath);
      return new File(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Synchronously copy this file. Returns a [File] instance for the copied file.
  ///
  /// If [newPath] identifies an existing file, that file is replaced.
  /// If [newPath] identifies an existing directory the operation fails and an exception is thrown.
  @override
  File copySync(String newPath) {
    try {
      _fs.copyFileSync(path, newPath);
      return new File(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Creates the file. Existing files are left untouched by [create].
  /// Returns a [Future] that completes with the file when it has been created.
  ///
  /// If [recursive] is `false`, the file is created only if all directories in its [path] exist.
  /// If [recursive] is `true`, all non-existing path components are created.
  @override
  Future<File> create({bool recursive = false}) async {
    if (await exists()) return this;
    if (recursive) await parent.create(recursive: true);
    return writeAsBytes(const []);
  }

  /// Synchronously creates the file. Existing files are left untouched by [createSync].
  /// Throws a [FileSystemException] if the operation fails.
  ///
  /// If [recursive] is `false`, the file is created only if all directories in its [path] exist.
  /// If [recursive] is `true`, all non-existing path components are created.
  @override
  void createSync({bool recursive = false}) {
    if (existsSync()) return;
    if (recursive) parent.createSync(recursive: true);
    writeAsBytesSync(const []);
  }

  /// Deletes this file.
  @override
  Future<File> delete({bool recursive = false}) async {
    try {
      await fn1cb0ToFuture(_fs.unlink, path);
      return this;
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, path);
    }
  }

  /// Synchronously deletes this file.
  @override
  void deleteSync({bool recursive = false}) {
    try { _fs.unlinkSync(path); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Gets the last-accessed time of the file.
  ///
  /// Returns a [Future] that completes with the date and time when the file was last accessed.
  /// If the information is unavailable the future completes with a [FileSystemException].
  @override
  Future<DateTime> lastAccessed() async {
    var stats = await stat();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.accessed;
  }

  /// Synchronously gets the last-accessed time of the file.
  ///
  /// Returns the date and time when the file was last accessed, if the information is available.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  DateTime lastAccessedSync() {
    var stats = statSync();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.accessed;
  }

  /// Gets the last-modified time of the file.
  ///
  /// Returns a [Future] that completes with the date and time when the file was last modified.
  /// If the information is unavailable the future completes with a [FileSystemException].
  @override
  Future<DateTime> lastModified() async {
    var stats = await stat();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.modified;
  }

  /// Synchronously gets the last-modified time of the file.
  ///
  /// Returns the date and time when the file was last modified, if the information is available.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  DateTime lastModifiedSync() {
    var stats = statSync();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.modified;
  }

  /// Gets the length of the file. Returns a [Future] that completes with the length in bytes.
  @override
  Future<int> length() async {
    var stats = await stat();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.size;
  }

  /// Synchronously gets the length of the file. Throws a [FileSystemException] if the operation fails.
  @override
  int lengthSync() {
    var stats = statSync();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    return stats.size;
  }

  /// Opens the file for random access operations.
  ///
  /// Returns a [Future] that completes with the opened [RandomAccessFile].
  /// [RandomAccessFile]s must be closed using the [RandomAccessFile.close] method.
  ///
  /// Files can be opened in three modes:
  /// - [READ]: open the file for reading.
  /// - [WRITE]: open the file for both reading and writing and truncate the file to length zero. If the file does not exist the file is created.
  /// - [APPEND]: same as [WRITE] except that the file is not truncated.
  @override
  Future<RandomAccessFile> open({f.FileMode mode = READ}) async =>
    new RandomAccessFile._internal(await fn2cb1ToFuture(_fs.open, path, _asFlags(mode)), path);

  /// Creates a new independent [Stream] for the contents of this file.
  ///
  /// If [start] is present, the file will be read from byte-offset start. Otherwise from the beginning (index 0).
  /// If [end] is present, only up to byte-index end will be read. Otherwise, until end of file.
  ///
  /// In order to make sure that system resources are freed, the stream must be read to completion or the subscription on the stream must be cancelled.
  @override
  Stream<List<int>> openRead([int start, int end]) => throw new UnimplementedError('File.openRead');

  /// Synchronously opens the file for random access operations.
  ///
  /// The result is a [RandomAccessFile] on which random access operations can be performed.
  /// Opened [RandomAccessFile]s must be closed using the [RandomAccessFile.close] method.
  ///
  /// Throws a [FileSystemException] if the operation fails.
  @override
  RandomAccessFile openSync({f.FileMode mode = READ}) =>
    new RandomAccessFile._internal(_fs.openSync(path, _asFlags(mode)), path);

  /// Creates a new independent [IOSink] for the file. The [IOSink] must be closed when no longer used, to free system resources.
  ///
  /// An [IOSink] for a file can be opened in two modes:
  /// - [WRITE]: truncates the file to length zero.
  /// - [APPEND]: sets the initial write position to the end of the file.
  ///
  /// When writing strings through the returned [IOSink] the encoding specified using [encoding] will be used.
  /// The returned [IOSink] has an `encoding` property which can be changed after the [IOSink] has been created.
  @override
  IOSink openWrite({Encoding encoding = UTF8, f.FileMode mode = WRITE}) => throw new UnimplementedError('File.openWrite');

  /// Reads the entire file contents as as a list of bytes.
  /// Returns a [Future] that completes with the list of bytes once the file contents has been read.
  @override
  Future<List<int>> readAsBytes() async {
    try { return await fn1cb1ToFuture<Uint8List>(_fs.readFile, path); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Synchronously reads the entire file contents as as a list of bytes.
  /// Throws a [FileSystemException] if the operation fails.
  @override
  List<int> readAsBytesSync() {
    try { return _fs.readFileSync(path); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Reads the entire file contents as lines of text using the given [encoding].
  /// Returns a [Future] that completes with the lines of text once the file contents has been read.
  @override
  Future<List<String>> readAsLines({Encoding encoding = UTF8}) async => (await readAsString(encoding: encoding)).split(new RegExp(r'\r?\n'));

  /// Synchronously reads the entire file contents as lines of text using the given [encoding].
  /// Throws a [FileSystemException] if the operation fails.
  @override
  List<String> readAsLinesSync({Encoding encoding = UTF8}) => readAsStringSync(encoding: encoding).split(new RegExp(r'\r?\n'));

  /// Reads the entire file contents as a string using the given [encoding].
  /// Returns a [Future] that completes with the string once the file contents has been read.
  @override
  Future<String> readAsString({Encoding encoding = UTF8}) async {
    try { return await fn2cb1ToFuture<String>(_fs.readFile, path, BufferEncoding.fromEncoding(encoding)); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Synchronously reads the entire file contents as a string using the given [encoding].
  /// Throws a [FileSystemException] if the operation fails.
  @override
  String readAsStringSync({Encoding encoding = UTF8}) {
    try { return _fs.readFileSync(path, BufferEncoding.fromEncoding(encoding)); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Renames this file. Completes with a [File] instance for the renamed file.
  @override
  Future<File> rename(String newPath) async {
    try {
      await fn2cb0ToFuture(_fs.rename, path, newPath);
      return new File(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Synchronously renames this file. Returns a [File] instance for the renamed file.
  @override
  File renameSync(String newPath) {
    try {
      _fs.renameSync(path, newPath);
      return new File(newPath);
    }

    on JSError catch (error) {
      throw new f.FileSystemException(error.message, newPath);
    }
  }

  /// Modifies the time the file was last accessed.
  /// Throws a [FileSystemException] if the time cannot be set.
  @override
  Future<Null> setLastAccessed(DateTime time) async {
    var stats = await stat();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    try { return await fn3cb0ToFuture(_fs.utimes, path, time, stats.modified); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Synchronously modifies the time the file was last accessed.
  /// Throws a [FileSystemException] if the time cannot be set.
  @override
  void setLastAccessedSync(DateTime time) {
    var stats = statSync();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    try { _fs.utimesSync(path, time, stats.modified); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Modifies the time the file was last modified.
  /// Throws a [FileSystemException] if the time cannot be set.
  @override
  Future<Null> setLastModified(DateTime time) async {
    var stats = await stat();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    try { return await fn3cb0ToFuture(_fs.utimes, path, stats.accessed, time); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Synchronously modifies the time the file was last modified.
  /// Throws a [FileSystemException] if the time cannot be set.
  @override
  void setLastModifiedSync(DateTime time) {
    var stats = statSync();
    if (stats.type == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('File not found', path);
    try { _fs.utimesSync(path, stats.accessed, time); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Returns a string representation of this object.
  @override
  String toString() => 'File("$path")';

  /// Writes a list of bytes to a file.
  ///
  /// Opens the file, writes the list of bytes in the given encoding, and closes the file.
  /// By default [writeAsBytes] creates the file for writing and truncates the file if it already exists.
  /// In order to append the bytes to an existing file, pass [APPEND] as the optional mode parameter.
  /// If the [flush] argument is set to `true` data written will be flushed to the file system before returning.
  ///
  /// Returns a [Future] that completes with this [File] object once the entire operation has completed.
  @override
  Future<File> writeAsBytes(List<int> bytes, {bool flush = false, f.FileMode mode = WRITE}) async {
    var fd = await open(mode: mode);
    await fd.writeFrom(bytes);
    if (flush) await fd.flush();
    await fd.close();
    return this;
  }

  /// Synchronously writes a list of bytes to a file.
  ///
  /// Opens the file, writes the list of bytes in the given encoding, and closes the file.
  /// By default [writeAsBytesSync] creates the file for writing and truncates the file if it already exists.
  /// In order to append the bytes to an existing file, pass [APPEND] as the optional mode parameter.
  /// If the [flush] argument is set to `true` data written will be flushed to the file system before returning.
  ///
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void writeAsBytesSync(List<int> bytes, {bool flush = false, f.FileMode mode = WRITE}) {
    var fd = openSync(mode: mode)..writeFromSync(bytes);
    if (flush) fd.flushSync();
    fd.closeSync();
  }

  /// Writes a string to a file.
  ///
  /// Opens the file, writes the string in the given encoding, and closes the file.
  /// By default [writeAsString] creates the file for writing and truncates the file if it already exists.
  /// In order to append the bytes to an existing file, pass [APPEND] as the optional mode parameter.
  /// If the [flush] argument is set to `true` data written will be flushed to the file system before returning.
  ///
  /// Returns a [Future] that completes with this [File] object once the entire operation has completed.
  @override
  Future<File> writeAsString(String contents, {Encoding encoding = UTF8, bool flush = false, f.FileMode mode = WRITE}) =>
    writeAsBytes(encoding.encode(contents), flush: flush, mode: mode);

  /// Synchronously writes a string to a file.
  ///
  /// Opens the file, writes the string in the given encoding, and closes the file.
  /// By default [writeAsStringSync] creates the file for writing and truncates the file if it already exists.
  /// In order to append the bytes to an existing file, pass [APPEND] as the optional mode parameter.
  /// If the [flush] argument is set to `true` data written will be flushed to the file system before returning.
  ///
  /// Throws a [FileSystemException] if the operation fails.
  @override
  void writeAsStringSync(String contents, {Encoding encoding = UTF8, bool flush = false, f.FileMode mode = WRITE}) =>
    writeAsBytesSync(encoding.encode(contents), flush: flush, mode: mode);

  /// Converts the specified [mode] to a string value.
  /// Throws an [ArgumentError] if the [mode] is not supported.
  String _asFlags(f.FileMode mode) {
    switch (mode) {
      case APPEND: return 'a+';
      case READ: return 'r';
      case WRITE: return 'w+';
      case WRITE_ONLY: return 'w';
      case WRITE_ONLY_APPEND: return 'a';
      default: throw new ArgumentError.value(mode);
    }
  }
}
