part of nodejs_interop.io;

/// The common super class for [File], [Directory], and [Link] objects.
abstract class FileSystemEntity implements f.FileSystemEntity {

  /// A reference to the `fs` module.
  static final FSModule _fs = require('fs');

  /// A reference to the `os` module.
  static final OSModule _os = require('os');

  /// Creates a new file system entity.
  FileSystemEntity._internal(this.path);

  /// Tests if [watch] is supported on the current system.
  static bool get isWatchSupported {
    if (!const Platform().isMacOS) return true;
    var version = _os.release().split('.').map((item) => int.parse(item, radix: 10, onError: (_) => -1)).toList();
    return version.first >= 10 && version[1] >= 7;
  }

  /// Checks whether two paths refer to the same object in the file system.
  /// Returns a [Future] that completes with the [bool] result.
  ///
  /// Comparing a link to its target returns `false`, as does comparing two links that point to the same target.
  /// To check the target of a link, use [Link.target] explicitly to fetch it.
  /// Directory links appearing inside a path are followed, though, to find the file system object.
  ///
  /// Completes the returned future with an error if one of the paths points to an object that does not exist.
  static Future<bool> identical(String path1, String path2) async {
    var type1 = await type(path1);
    if (type1 == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('Entity not found', path1);

    var type2 = await type(path2);
    if (type2 == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('Entity not found', path2);

    var path = const FileSystem().path;
    return type1 == type2 && path.canonicalize(path1) == path.canonicalize(path2);
  }

  /// Synchronously checks whether two paths refer to the same object in the file system.
  ///
  /// Comparing a link to its target returns `false`, as does comparing two links that point to the same target.
  /// To check the target of a link, use [Link.target] explicitly to fetch it.
  /// Directory links appearing inside a path are followed, though, to find the file system object.
  ///
  /// Throws an error if one of the paths points to an object that does not exist.
  static bool identicalSync(String path1, String path2) {
    var type1 = typeSync(path1);
    if (type1 == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('Entity not found', path1);

    var type2 = typeSync(path2);
    if (type2 == f.FileSystemEntityType.NOT_FOUND) throw new f.FileSystemException('Entity not found', path2);

    var path = const FileSystem().path;
    return type1 == type2 && path.canonicalize(path1) == path.canonicalize(path2);
  }

  /// Checks whether the file system entity at [path] is a directory.
  static Future<bool> isDirectory(String path) async => await type(path) == f.FileSystemEntityType.DIRECTORY;

  /// Syncronously checks whether the file system entity at [path] is a directory.
  static bool isDirectorySync(String path) => typeSync(path) == f.FileSystemEntityType.DIRECTORY;

  /// Checks whether the file system entity at [path] is a file.
  static Future<bool> isFile(String path) async => await type(path) == f.FileSystemEntityType.FILE;

  /// Syncronously checks whether the file system entity at [path] is a file.
  static bool isFileSync(String path) => typeSync(path) == f.FileSystemEntityType.FILE;

  /// Checks whether the file system entity at [path] is a link.
  static Future<bool> isLink(String path) async => await type(path) == f.FileSystemEntityType.LINK;

  /// Syncronously checks whether the file system entity at [path] is a link.
  static bool isLinkSync(String path) => typeSync(path) == f.FileSystemEntityType.LINK;

  /// Removes the final path component of a [path], using the platform's path separator to split the path.
  ///
  /// Will not remove the root component of a Windows path, like `"C:\"` or `"\\server_name\"`.
  /// Ignores trailing path separators, and leaves no trailing path separators.
  static String parentOf(String path) => const FileSystem().path.dirname(path);

  /// Finds the type of file system entity that a [path] points to.
  ///
  /// [f.FileSystemEntityType.LINK] will only be returned if [followLinks] is `false`, and [path] points to a link.
  /// If the [path] does not point to a file system object or an error occurs then [f.FileSystemEntityType.NOT_FOUND] is returned.
  static Future<f.FileSystemEntityType> type(String path, {bool followLinks = true}) async => (await FileStat.stat(path)).type;

  /// Syncronously finds the type of file system entity that a [path] points to.
  ///
  /// [f.FileSystemEntityType.LINK] will only be returned if [followLinks] is `false`, and [path] points to a link.
  /// If the [path] does not point to a file system object or an error occurs then [f.FileSystemEntityType.NOT_FOUND] is returned.
  static f.FileSystemEntityType typeSync(String path, {bool followLinks = true}) => FileStat.statSync(path).type;

  /// The part of this entity's path after the last separator.
  @override
  String get basename => fileSystem.path.basename(path);

  /// The part of this entity's path before the last separator.
  @override
  String get dirname => fileSystem.path.dirname(path);

  /// The file system responsible for this entity.
  @override
  FileSystem get fileSystem => const FileSystem();

  /// Value indicating whether this entity's path is absolute.
  @override
  bool get isAbsolute => fileSystem.path.isAbsolute(path);

  /// The directory containing this entity.
  @override
  Directory get parent => new Directory(dirname);

  /// This entity's path.
  @override
  final String path;

  /// Checks whether the file system entity with this [path] exists.
  @override
  Future<bool> exists() async {
    try {
      await fn1cb0ToFuture(_fs.access, path);
      return true;
    }

    on JSError {
      return false;
    }
  }

  /// Synchronously checks whether the file system entity with this [path] exists.
  @override
  bool existsSync() => _fs.existsSync(path);

  /// Resolves the [path] of this entity relative to the current working directory,
  /// resolving all symbolic links on the path and resolving all `..` and `.` path segments.
  @override
  Future<String> resolveSymbolicLinks() async {
    try { return await fn1cb1ToFuture<String>(_fs.realpath, path); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Synchronously resolves the [path] of this entity relative to the current working directory,
  /// resolving all symbolic links on the path and resolving all `..` and `.` path segments.
  @override
  String resolveSymbolicLinksSync() {
    try { return _fs.realpathSync(path); }
    on JSError catch (error) { throw new f.FileSystemException(error.message, path); }
  }

  /// Calls the operating system's `stat()` function on this entity's [path].
  ///
  /// Returns a [Future] object containing the [FileStat] data returned by `stat()`.
  /// If the call fails, completes the future with a [FileStat] object with `.type` set to [f.FileSystemEntityType.NOT_FOUND] and the other fields invalid.
  @override
  Future<FileStat> stat() => FileStat.stat(path);

  /// Synchronously calls the operating system's `stat()` function on this entity's [path].
  ///
  /// Returns a [FileStat] object containing the data returned by `stat()`.
  /// If the call fails, returns a [FileStat] object with `.type` set to [f.FileSystemEntityType.NOT_FOUND] and the other fields invalid.
  @override
  FileStat statSync() => FileStat.statSync(path);

  /// Starts watching this entity for changes.
  ///
  /// The implementation uses platform-dependent event-based APIs for receiving file-system notifications, thus behavior depends on the platform.
  /// The system will start listening for events once the returned [Stream] is being listened to, not when the call to [watch] is issued.
  ///
  /// The returned value is an endless broadcast [Stream] that only stops when the [Stream] is canceled, or when the entity being watches is deleted.
  /// Use [events] to specify what events to listen for.
  ///
  /// Note that on most platforms, a [FileSystemEvent.MOVE] event is emitted whenever a filename appears or disappears in the directory.
  @override
  Stream<FileSystemEvent> watch({int events = FileSystemEvent.ALL, bool recursive = false}) {
    var watcher = _fs.watch(path, new WatchOptions(recursive: recursive));
    var controller = new StreamController<FileSystemEvent>.broadcast();
    controller.onCancel = () {
      if (!controller.hasListener) watcher.close();
    };

    watcher
      ..on('error', allowInterop((error) => controller..addError(new f.FileSystemException(error.message))..close()))
      ..on('change', allowInterop((eventType, filename) async {
        var type = eventType == 'rename' ? FileSystemEvent.MOVE : FileSystemEvent.MODIFY;
        if ((events & type) != 0) controller.add(
          type == FileSystemEvent.MOVE ?
          new FileSystemMoveEvent._internal(filename, isDirectory: await isDirectory(filename)) :
          new FileSystemModifyEvent._internal(filename, isDirectory: await isDirectory(filename))
        );
      }));

    return controller.stream;
  }
}
