part of nodejs_interop.io;

/// Base event class emitted by [FileSystemEntity.watch].
class FileSystemEvent implements f.FileSystemEvent {

  /// Bitfield for [FileSystemEntity.watch], to enable [FileSystemCreateEvent]s.
  static const int CREATE = 1 << 0; // ignore: constant_identifier_names

  /// Bitfield for [FileSystemEntity.watch], to enable [FileSystemModifyEvent]s.
  static const int MODIFY = 1 << 1; // ignore: constant_identifier_names

  /// Bitfield for [FileSystemEntity.watch], to enable [FileSystemDeleteEvent]s.
  static const int DELETE = 1 << 2; // ignore: constant_identifier_names

  /// Bitfield for [FileSystemEntity.watch], to enable [FileSystemMoveEvent]s.
  static const int MOVE = 1 << 3; // ignore: constant_identifier_names

  /// Bitfield for [FileSystemEntity.watch], for enabling all of [CREATE], [MODIFY], [DELETE] and [MOVE].
  static const int ALL = CREATE | DELETE | MODIFY | MOVE; // ignore: constant_identifier_names

  /// Creates a new file system event.
  FileSystemEvent._internal(this.type, this.path, {this.isDirectory = false});

  /// Value indicating whether the event target was a directory.
  @override
  final bool isDirectory;

  /// The path that triggered the event.
  @override
  final String path;

  /// The type of event.
  @override
  final int type;
}

/// File system event for newly created file system objects.
class FileSystemCreateEvent extends FileSystemEvent {

  /// Creates a new [FileSystemEvent.CREATE] file system event.
  FileSystemCreateEvent._internal(String path, {bool isDirectory = false}):
    super._internal(FileSystemEvent.CREATE, path, isDirectory: isDirectory);

  /// Returns a string representation of this object.
  @override
  String toString() => 'FileSystemCreateEvent("$path")';
}

/// File system event for deletion of file system objects.
class FileSystemDeleteEvent extends FileSystemEvent {

  /// Creates a new [FileSystemEvent.DELETE] file system event.
  FileSystemDeleteEvent._internal(String path, {bool isDirectory = false}):
    super._internal(FileSystemEvent.DELETE, path, isDirectory: isDirectory);

  /// Returns a string representation of this object.
  @override
  String toString() => 'FileSystemDeleteEvent("$path")';
}

/// File system event for modifications of file system objects.
class FileSystemModifyEvent extends FileSystemEvent {

  /// Creates a new [FileSystemEvent.MODIFY] file system event.
  FileSystemModifyEvent._internal(String path, {this.contentChanged = true, bool isDirectory = false}):
    super._internal(FileSystemEvent.MODIFY, path, isDirectory: isDirectory);

  /// Value indicating whether the content was changed and not only the file attributes.
  final bool contentChanged;

  /// Returns a string representation of this object.
  @override
  String toString() => 'FileSystemModifyEvent("$path", contentChanged: $contentChanged)';
}

/// File system event for moving of file system objects.
class FileSystemMoveEvent extends FileSystemEvent {

  /// Creates a new [FileSystemEvent.MOVE] file system event.
  FileSystemMoveEvent._internal(String path, {this.destination, bool isDirectory = false}):
    super._internal(FileSystemEvent.MOVE, path, isDirectory: isDirectory);

  /// If the underlying implementation is able to identify the destination of the moved file, [destination] will be set.
  /// Otherwise, it will be a `null` reference.
  final String destination;

  /// Returns a string representation of this object.
  @override
  String toString() => 'FileSystemMoveEvent("$path", destination: "$destination")';
}
