part of nodejs_interop.io;

/// Information about the environment in which the current program is running.
///
/// [Platform] provides information such as the operating system, the hostname of the computer,
/// the value of environment variables, the path to the running program, and so on.
class Platform extends platform.Platform {

  /// A reference to the `os` module.
  static final OSModule _os = require('os');

  /// A reference to the `path` module.
  static final PathModule _path = require('path');

  /// Creates a new platform.
  const Platform();

  /// The environment for this process.
  /// It is an unmodifiable map whose content is retrieved from the operating system.
  ///
  /// Environment variables on Windows are case-insensitive. The keys of the map returned on Windows are therefore case-insensitive.
  /// On other platforms the keys of the returned map are case-sensitive.
  @override
  Map<String, String> get environment {
    var jsEnv = getProperties(process.env);
    return new UnmodifiableMapView(isWindows ? new EqualityMap.from(const CaseInsensitiveEquality(), jsEnv) : jsEnv);
  }

  /// The path of the executable used to run the script in this isolate.
  ///
  /// The path returned is the literal path used to run the script.
  /// To get the absolute path to the resolved executable use [resolvedExecutable].
  @override
  String get executable => process.argv0;

  /// The flags passed to the executable used to run the script in this isolate.
  @override
  List<String> get executableArguments => process.execArgv;

  /// The name of the current locale.
  /// Uses the [`os-locale`](https://www.npmjs.com/package/os-locale) package if available.
  @override
  String get localeName => loadLibrary<OSLocaleModule>(
    'os-locale', onError: (_) => throw new UnsupportedError('Not supported by Node.js: install the "os-locale" npm package')
  ).sync();

  /// The local hostname for the system.
  @override
  String get localHostname => _os.hostname();

  /// The number of processors of the machine.
  @override
  int get numberOfProcessors => _os.cpus().length;

  /// A string representing the operating system.
  @override
  String get operatingSystem {
    switch (process.platform) {
      case 'darwin': return 'macos';
      case 'win32': return 'windows';
      default: return 'linux';
    }
  }

  /// The value of the `--packages` flag passed to the executable used to run the script in this isolate.
  @override
  String get packageConfig => throw new UnsupportedError('Not supported by Node.js');

  /// The value of the `--package-root` flag passed to the executable used to run the script in this isolate.
  @override
  String get packageRoot => throw new UnsupportedError('Not supported by Node.js');

  /// The path separator used by the operating system to separate components in file paths.
  @override
  String get pathSeparator => _path.sep;

  /// The path of the executable used to run the script in this isolate after it has been resolved by the OS.
  /// This is the absolute path, with all symlinks resolved, to the executable used to run the script.
  @override
  String get resolvedExecutable => process.execPath;

  /// The absolute URI of the script being run in this isolate.
  @override
  Uri get script => new Uri.file(filename, windows: isWindows);

  /// When `stdin` is connected to a terminal, whether ANSI codes are supported.
  @override
  bool get stdinSupportsAnsi => throw new UnsupportedError('Not supported by Node.js');

  /// When `stdout` is connected to a terminal, whether ANSI codes are supported.
  /// Uses the [`supports-color`](https://www.npmjs.com/package/supports-color) package if available.
  @override
  bool get stdoutSupportsAnsi => loadLibrary<SupportsColorModule>(
    'supports-color', onError: (_) => throw new UnsupportedError('Not supported by Node.js: install the "supports-color" npm package')
  ).stdout != false;

  /// The version of the current Node.js runtime.
  /// The value is a [semantic versioning](http://semver.org) string representing the version of the current Node.js runtime.
  @override
  String get version => process.version.substring(1);
}
