part of nodejs_interop.io;

/// An internet address.
///
/// This object holds an internet address. If this internet address is the result of a DNS lookup,
/// the address also holds the hostname used to make the lookup. An Internet address combined with a port number
/// represents an endpoint to which a socket can connect or a listening socket can bind.
class InternetAddress implements io.InternetAddress {

  /// A reference to the `dns` module.
  static final DnsModule _dns = require('dns');

  /// A reference to the `net` module.
  static final NetModule _net = require('net');

  /// Creates a new Internet address.
  InternetAddress(String address): this._internal(address);

  /// Creates a new Internet address.
  /// Throws an [ArgumentError] if the `address` is not a numeric IPv4 (dotted-decimal notation) or IPv6 (hexadecimal representation).
  InternetAddress._internal(this.address, [String host]): host = host ?? address {
    if (_net.isIP(address) == 0) throw new ArgumentError.value(address);
  }

  /// IP version 4 any address.
  /// Use this address when listening on all adapters IP addresses using IP version 4 (IPv4).
  static InternetAddress get ANY_IP_V4 => new InternetAddress('0.0.0.0'); // ignore: non_constant_identifier_names, prefer_constructors_over_static_methods

  /// IP version 6 any address.
  /// Use this address when listening on all adapters IP addresses using IP version 6 (IPv6).
  static InternetAddress get ANY_IP_V6 => new InternetAddress('::'); // ignore: non_constant_identifier_names, prefer_constructors_over_static_methods

  /// IP version 4 loopback address.
  /// Use this address when listening on or connecting to the loopback adapter using IP version 4 (IPv4).
  static InternetAddress get LOOPBACK_IP_V4 => new InternetAddress('127.0.0.1'); // ignore: non_constant_identifier_names, prefer_constructors_over_static_methods

  /// IP version 6 loopback address.
  /// Use this address when listening on or connecting to the loopback adapter using IP version 6 (IPv6).
  static InternetAddress get LOOPBACK_IP_V6 => new InternetAddress('::1'); // ignore: non_constant_identifier_names, prefer_constructors_over_static_methods

  /// The numeric address of the host.
  /// For IPv4 addresses this is using the dotted-decimal notation. For IPv6 it is using the hexadecimal representation.
  @override
  final String address;

  /// The host used to lookup the address. If there is no host associated with the address this returns the numeric [address].
  @override
  final String host;

  /// Value indicating whether the [InternetAddress]'s scope is a link-local.
  /// Uses the [`ipaddr.js`](https://www.npmjs.com/package/ipaddr.js) package if available.
  @override
  bool get isLinkLocal => loadLibrary<IPAddrModule>(
    'ipaddr.js', onError: (_) => throw new UnsupportedError('Not supported by Node.js: install the "ipaddr.js" npm package')
  ).parse(address).range() == 'linkLocal';

  /// Value indicating whether the [InternetAddress] is a loopback address.
  /// Uses the [`ipaddr.js`](https://www.npmjs.com/package/ipaddr.js) package if available.
  @override
  bool get isLoopback => loadLibrary<IPAddrModule>(
    'ipaddr.js', onError: (_) => throw new UnsupportedError('Not supported by Node.js: install the "ipaddr.js" npm package')
  ).parse(address).range() == 'loopback';

  /// Value indicating whether the [InternetAddress]'s scope is multicast.
  /// Uses the [`ipaddr.js`](https://www.npmjs.com/package/ipaddr.js) package if available.
  @override
  bool get isMulticast => loadLibrary<IPAddrModule>(
    'ipaddr.js', onError: (_) => throw new UnsupportedError('Not supported by Node.js: install the "ipaddr.js" npm package')
  ).parse(address).range() == 'multicast';

  /// The raw address of this [InternetAddress]. The result is either a 4 or 16 byte long list.
  /// The returned list is a copy, making it possible to change the list without modifying the [InternetAddress].
  ///
  /// Uses the [`ipaddr.js`](https://www.npmjs.com/package/ipaddr.js) package if available.
  @override
  List<int> get rawAddress => loadLibrary<IPAddrModule>(
    'ipaddr.js', onError: (_) => throw new UnsupportedError('Not supported by Node.js: install the "ipaddr.js" npm package')
  ).parse(address).toByteArray();

  /// The type of IP protocol of this [InternetAddress].
  @override
  io.InternetAddressType get type {
    switch (_net.isIP(address)) {
      case 4: return io.InternetAddressType.IP_V4;
      case 6: return io.InternetAddressType.IP_V6;
      default: return io.InternetAddressType.ANY;
    }
  }

  @override
  set type(io.InternetAddressType value) => throw new UnsupportedError('The type of the internet address is unmodifiable');

  /// Lookup a [host], returning a [Future] of a list of [InternetAddress] objects.
  ///
  /// If [type] is [io.InternetAddressType.ANY], it will lookup both IP version 4 (IPv4) and IP version 6 (IPv6) addresses.
  /// If [type] is either [io.InternetAddressType.IP_V4] or [io.InternetAddressType.IP_V6] it will only lookup addresses of the specified type.
  static Future<List<InternetAddress>> lookup(String host, {io.InternetAddressType type: io.InternetAddressType.ANY}) async {
    try {
      var addresses = await fn2cb1ToFuture<List<IPAddress>>(_dns.lookup, host, new LookupOptions(all: true, verbatim: true));
      return addresses.map((address) => new InternetAddress._internal(address.address, host)).toList();
    }

    on JSError catch (error) {
      throw new io.SocketException(error.message);
    }
  }

  /// Perform a reverse DNS lookup on the [address], creating a new [InternetAddress] where the [host] field set to the result.
  @override
  Future<InternetAddress> reverse() async {
    try {
      var hosts = await fn1cb1ToFuture<List<String>>(_dns.reverse, address);
      if (hosts.isEmpty) throw new io.SocketException('Failed reverse host lookup', address: this);
      return new InternetAddress._internal(address, hosts.first);
    }

    on JSError catch (error) {
      throw new io.SocketException(error.message, address: this);
    }
  }

  /// Returns a string representation of this object.
  @override
  String toString() => 'InternetAddress("$address", ${type.name})';
}
