part of nodejs_interop.io;

/// Represents an active network interface on the current system.
class NetworkInterface implements io.NetworkInterface {

  /// A reference to the `os` module.
  static final OSModule _os = require('os');

  /// Creates a new network interface.
  NetworkInterface._internal(this.name, this.addresses);

  /// Value indicating whether the [list] method is supported.
  static bool get listSupported => true;

  /// Queries the system for network interfaces.
  ///
  /// If [includeLoopback] is `true`, the returned list will include the loopback device.
  /// If [includeLinkLocal] is `true`, the list of addresses of the returned network interfaces, may include link local addresses.
  static Future<List<NetworkInterface>> list({
    bool includeLinkLocal = false,
    bool includeLoopback = false,
    io.InternetAddressType type = io.InternetAddressType.ANY
  }) {
    var interfaces = <NetworkInterface>[];
    var nativeInterfaces = getProperties(_os.networkInterfaces());

    for (var name in nativeInterfaces.keys) {
      var addresses = <InternetAddress>[];
      for (NetworkAddress nativeAddress in nativeInterfaces[name]) {
        var address = new InternetAddress(nativeAddress.address);
        if (!includeLinkLocal && address.isLinkLocal) continue;
        if (!includeLoopback && address.isLoopback) continue;
        if (type != io.InternetAddressType.ANY && address.type != type) continue;
        addresses.add(address);
      }

      if (addresses.isNotEmpty) interfaces.add(new NetworkInterface._internal(name, addresses));
    }

    return new Future<List<NetworkInterface>>.value(interfaces);
  }

  /// The list of internet addresses currently bound to this network interface.
  @override
  final List<InternetAddress> addresses;

  /// The index of this network interface.
  @override
  String get index => throw new UnsupportedError('Not supported by Node.js');

  /// The name of this network interface.
  @override
  final String name;

  /// Returns a string representation of this object.
  @override
  String toString() => 'NetworkInterface("$name", $addresses)';
}
