part of nodejs_interop.node;

/// Represents a spawned child process.
@JS()
abstract class ChildProcess extends EventEmitter {

  /// A reference to this child's IPC channel. If no IPC channel currently exists, this property is `null`.
  external dynamic get channel;

  /// Value indicating whether it is still possible to send and receive messages from this child process.
  /// When [connected] is `false`, it is no longer possible to send or receive messages.
  external bool get connected;

  /// Value indicating whether this child process successfully received a signal from [kill].
  /// The [killed] property does not indicate that the child process has been terminated.
  external bool get killed;

  /// The process identifier (PID) of this child process.
  external int get pid;

  /// A [Readable] stream that represents this child process's `stderr`.
  /// [stderr] is an alias for `stdio[2]`. Both properties will refer to the same value.
  external Readable get stderr;

  /// A [Writable] stream that represents this child process's `stdin`.
  /// [stdin] is an alias for `stdio[0]`. Both properties will refer to the same value.
  external Writable get stdin;

  /// A sparse array of pipes to this child process, corresponding with positions in the `stdio` option
  /// passed to [ChildProcessModule.spawn] that have been set to the value `"pipe"`.
  ///
  /// Note that `stdio[0]`, `stdio[1]`, and `stdio[2]` are also available as [stdin], [stdout], and [stderr], respectively.
  external List get stdio;

  /// A [Readable] stream that represents this child process's `stdout`.
  /// [stdout] is an alias for `stdio[1]`. Both properties will refer to the same value.
  external Readable get stdout;

  /// Closes the IPC channel between parent and child, allowing the child to exit gracefully once there are no other connections keeping it alive.
  ///
  /// After calling this method the [connected] properties in both the parent and child (respectively) will be set to `false`,
  /// and it will be no longer possible to pass messages between the processes.
  external void disconnect();

  /// Sends a [signal] to this child process.
  /// Note that while the method is called `kill`, the signal delivered to the child process may not actually terminate the process.
  external void kill([String signal = 'SIGTERM']);

  /// When an IPC channel has been established between the parent and child (i.e. when using [ChildProcessModule.fork]),
  /// the [send] method can be used to send messages to the child process. When the child process is a Node.js instance,
  /// these messages can be received via the `process.on("message")` event.
  ///
  /// The message goes through JSON serialization and parsing. The resulting message might not be the same as what is originally sent.
  external bool send(message, [sendHandleOrOptions, optionsOrCallback, CompletionCallback0 callback]);
}

/// Options used to customize the sending of a message via an IPC channel.
@JS()
@anonymous
class SendOptions {

  /// Creates a new options object.
  external factory SendOptions({bool keepOpen = false});

  /// Value indicating whether the socket is kept open in the sending process.
  external bool get keepOpen;
  external set keepOpen(bool value);
}
