part of nodejs_interop.node;

/// The [`child_process`](https://nodejs.org/api/child_process.html) module.
@JS()
abstract class ChildProcessModule {

  /// Spawns a shell then executes the [command] within that shell, buffering any generated output.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  ///
  /// If a [callback] function is provided, it is called with the arguments `(error, stdout, stderr)`.
  /// The `stdout` and `stderr` arguments passed to the [callback] will contain the `stdout` and `stderr` output of the child process.
  external ChildProcess exec(String command, [optionsOrCallback, CompletionCallback2 callback]);

  /// Spawns the specified [executable] file, with command line arguments in `args`.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  ///
  /// If a [callback] function is provided, it is called with the arguments `(error, stdout, stderr)`.
  /// The `stdout` and `stderr` arguments passed to the [callback] will contain the `stdout` and `stderr` output of the child process.
  external ChildProcess execFile(String executable, [argsOrOptions, optionsOrCallback, CompletionCallback2 callback]);

  /// Spawns the specified [executable] file, with command line arguments in `args`. Returns the `stdout` from the command.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  ///
  /// This is a synchronous call and will block until the child process terminates.
  /// If the process times out, or has a non-zero exit code, this method will throw.
  external dynamic execFileSync(String executable, [argsOrOptions, SpawnOptions options]);

  /// Spawns a shell then executes the [command] within that shell, buffering any generated output. Returns the `stdout` from the command.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  ///
  /// This is a synchronous call and will block until the child process terminates.
  /// If the process times out, or has a non-zero exit code, this method will throw.
  external dynamic execSync(String command, [SpawnOptions options]);

  /// Spawns a Node.js process using the given [modulePath], with command line arguments in `args`.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  ///
  /// Returns a [ChildProcess] that has an additional IPC communication channel built-in allowing messages
  /// to be passed back and forth between the parent and child.
  external ChildProcess fork(String modulePath, [argsOrOptions, SpawnOptions options]);

  /// Spawns a new process using the given [executable], with command line arguments in `args`.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  external ChildProcess spawn(String executable, [argsOrOptions, SpawnOptions options]);

  /// Spawns a new process using the given [executable], with command line arguments in `args`.
  /// [SpawnOptions] may be passed to customize how the process is spawned.
  ///
  /// This is a synchronous call and will block until the child process terminates.
  external SpawnResult spawnSync(String executable, [argsOrOptions, SpawnOptions options]);
}

/// Options used to customize the spawning of a child process.
@JS()
@anonymous
class SpawnOptions {

  /// Creates a new options object.
  external factory SpawnOptions({
    String argv0,
    String cwd,
    bool detached = false,
    String encoding,
    env,
    List<String> execArgv,
    String execPath,
    int gid,
    input,
    killSignal = 'SIGTERM',
    int maxBuffer = 200 * 1024,
    shell,
    bool silent = false,
    stdio = 'pipe',
    int timeout = 0,
    int uid,
    bool windowsHide = false,
    bool windowsVerbatimArguments = false
  });

  /// The value of `argv[0]` sent to the child process.
  /// Only supported by the [ChildProcessModule.spawn] method.
  external String get argv0;
  external set argv0(String value);

  /// The working directory for the child process.
  external String get cwd;
  external set cwd(String value);

  /// Value indicating whether to prepare child to run independently of its parent process.
  /// Only supported by the [ChildProcessModule.spawn] method.
  external bool get detached;
  external set detached(bool value);

  /// The encoding used for decoding `stdout` and `stderr`.
  /// Not supported by the [ChildProcessModule.fork] and [ChildProcessModule.spawn] methods.
  external String get encoding;
  external set encoding(String value);

  /// The environment variables for the child process.
  external dynamic get env;
  external set env(value);

  /// The list of arguments passed to the executable.
  /// Only supported by the [ChildProcessModule.fork] method.
  external List<String> get execArgv;
  external set execArgv(List<String> value);

  /// The executable used to create the child process.
  /// Only supported by the [ChildProcessModule.fork] method.
  external String get execPath;
  external set execPath(String value);

  /// The group identity of the child process.
  external int get gid;
  external set gid(int value);

  /// The value which will be passed as `stdin` to the spawned process. Supplying this value will override `stdio[0]`.
  /// Only supported by the [ChildProcessModule]'s synchronous methods.
  external dynamic get input;
  external set input(value);

  /// The signal value to be used when the spawned process will be killed.
  /// Not supported by the [ChildProcessModule.fork] and [ChildProcessModule.spawn] methods.
  external dynamic get killSignal;
  external set killSignal(value);

  /// The largest amount of data in bytes allowed on `stdout` or `stderr`.
  /// Not supported by the [ChildProcessModule.fork] and [ChildProcessModule.spawn] methods.
  external int get maxBuffer;
  external set maxBuffer(int value);

  /// Value indicating whether to run the command inside a shell. A different shell can be specified as a string.
  /// Not supported by the [ChildProcessModule.execFile], [ChildProcessModule.execFileSync] and [ChildProcessModule.fork] methods.
  external dynamic get shell;
  external set shell(value);

  /// Value indicating whether `stdio` of the child process will be piped to the parent instead of be inherited from.
  /// Only supported by the [ChildProcessModule.fork] method.
  external bool get silent;
  external set silent(bool value);

  /// The `stdio` configuration of the child process.
  /// Not supported by the [ChildProcessModule.exec] and [ChildProcessModule.execFile] methods.
  external dynamic get stdio;
  external set stdio(value);

  /// The maximum amount of time, in milliseconds, the child process is allowed to run.
  /// Not supported by the [ChildProcessModule.fork] and [ChildProcessModule.spawn] methods.
  external int get timeout;
  external set timeout(int value);

  /// The user identity of the child process.
  external int get uid;
  external set uid(int value);

  /// Value indicating whether to hide the subprocess console window that would normally be created on Windows systems.
  /// Not supported by the [ChildProcessModule.fork] method.
  external bool get windowsHide;
  external set windowsHide(bool value);

  /// Value indicating whether no quoting or escaping of arguments is done on Windows.
  /// Not supported by the [ChildProcessModule.exec], [ChildProcessModule.execFileSync] and [ChildProcessModule.execSync] methods.
  external bool get windowsVerbatimArguments;
  external set windowsVerbatimArguments(bool value);
}

/// Represents the result of spawning a child process.
@JS()
abstract class SpawnResult {

  /// The error object if the child process failed or timed out.
  external JSError get error;

  /// The list of results from `stdio` output.
  external List get output;

  /// The process identifier (PID) of the child process.
  external int get pid;

  /// The signal used to kill the child process.
  external String get signal;

  /// The exit code of the child process.
  external int get status;

  /// The contents of `output[1]`.
  external dynamic get stderr;

  /// The contents of `output[2]`.
  external dynamic get stdout;
}
