part of nodejs_interop.node;

/// Options used to customize the resolution of a hostname.
@JS()
@anonymous
class ResolveOptions {

  /// Creates a new path object.
  external factory ResolveOptions({bool ttl = false});

  /// Value indicating whether to retrieve the Time-To-Live value (TTL), expressed in seconds, of each record.
  external bool get ttl;
  external set ttl(bool value);
}

/// An independent resolver for DNS requests.
@JS(r'$dns.Resolver')
class Resolver extends _Resolver {

  /// Creates a new DNS resolver.
  external Resolver();

  /// Cancel all outstanding DNS queries made by this resolver.
  /// The corresponding callbacks will be called with an error with code `"ECANCELLED"`.
  external void cancel();
}

/// A resolver for DNS requests.
@JS()
abstract class _Resolver {

  /// Returns a list of IP addresses that are currently configured for DNS resolution.
  /// A string will include a port section if a custom port is used.
  external List<String> getServers();

  /// Uses the DNS protocol to resolve a [hostname] into a list of the resource records.
  ///
  /// The [callback] function has arguments `(err, records)`. When successful, records will be a list of resource records.
  /// The type and structure of individual results varies based on `rrtype` (e.g. `"A"`, `"AAAA"`, `"CNAME"`, ...).
  external void resolve(String hostname, rrtypeOrCallback, [CompletionCallback1 callback]);

  /// Uses the DNS protocol to resolve a IPv4 addresses (`A` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will contain a list of IPv4 addresses.
  external void resolve4(String hostname, optionsOrCallback, [CompletionCallback1<List> callback]);

  /// Uses the DNS protocol to resolve a IPv6 addresses (`AAAA` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will contain a list of IPv6 addresses.
  external void resolve6(String hostname, optionsOrCallback, [CompletionCallback1<List> callback]);

  /// Uses the DNS protocol to resolve all records (also known as `ANY` or `*` query).
  /// The `ret` argument passed to the [callback] function will be a list containing various types of [DnsRecord] objects.
  /// Each object has a property `type` that indicates the type of the current record.
  external void resolveAny(String hostname, CompletionCallback1<List<DnsRecord>> callback);

  /// Uses the DNS protocol to resolve `CNAME` records for the [hostname].
  /// The `addresses` argument passed to the [callback] function will contain a list of canonical name records available for the [hostname].
  external void resolveCname(String hostname, CompletionCallback1<List<String>> callback);

  /// Uses the DNS protocol to resolve mail exchange records (`MX` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will contain a list of [MXRecord] objects.
  external void resolveMx(String hostname, CompletionCallback1<List<MXRecord>> callback);

  /// Uses the DNS protocol to resolve regular expression based records (`NAPTR` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will contain a list of [NaptrRecord] objects.
  external void resolveNaptr(String hostname, CompletionCallback1<List<NaptrRecord>> callback);

  /// Uses the DNS protocol to resolve name server records (`NS` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will contain a list of name server records available for [hostname].
  external void resolveNs(String hostname, CompletionCallback1<List<String>> callback);

  /// Uses the DNS protocol to resolve pointer records (`PTR` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will be a list of strings containing the reply records.
  external void resolvePtr(String hostname, CompletionCallback1<List<String>> callback);

  /// Uses the DNS protocol to resolve a start of authority record (`SOA` record) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will be a [SoaRecord].
  external void resolveSoa(String hostname, CompletionCallback1<SoaRecord> callback);

  /// Uses the DNS protocol to resolve service records (`SRV` records) for the [hostname].
  /// The `addresses` argument passed to the [callback] function will be a list of [SrvRecord] objects.
  external void resolveSrv(String hostname, CompletionCallback1<List<SrvRecord>> callback);

  /// Uses the DNS protocol to resolve text queries (`TXT` records) for the [hostname].
  /// The `records` argument passed to the [callback] function is a two-dimensional list of the text records available for [hostname].
  external void resolveTxt(String hostname, CompletionCallback1<List<List<String>>> callback);

  /// Performs a reverse DNS query that resolves an IPv4 or IPv6 [address] to a list of hostnames.
  external void reverse(String address, CompletionCallback1<List<String>> callback);

  /// Sets the IP address and port of [servers] to be used when performing DNS resolution.
  /// If the port is the default DNS port (i.e. `53`) it can be omitted.
  external void setServers(List<String> servers);
}
