part of nodejs_interop.node;

/// The [`fs`](https://nodejs.org/api/fs.html) module.
@JS()
abstract class FSModule {

  /// Commonly used constants for file system operations.
  external FSConstants get constants;

  /// Tests a user's permissions for the file or directory specified by [path].
  ///
  /// The `mode` argument is an optional integer that specifies the accessibility checks to be performed.
  /// The [callback] argument is a function that is invoked with a possible error argument.
  /// If any of the accessibility checks fail, the error argument will be populated.
  external void access(path, modeOrCallback, [CompletionCallback0 callback]);

  /// Synchronously tests a user's permissions for the file or directory specified by [path].
  /// The `mode` argument is an optional integer that specifies the accessibility checks to be performed.
  external void accessSync(path, [int mode]);

  /// Appends [data] to a [file]. [data] can be a string or a [Uint8List].
  ///
  /// The `options` argument can be a string specifying an encoding or a [WriteFileOptions] indicating the character encoding of the data.
  /// The `encoding` option is ignored if [data] is a [Uint8List].
  ///
  /// If a file descriptor is specified as the [file], it will not be closed automatically.
  external void appendFile(file, data, optionsOrCallback, [CompletionCallback0 callback]);

  /// Synchronously appends [data] to a [file]. [data] can be a string or a [Uint8List].
  ///
  /// The [options] argument can be a string specifying an encoding or a [WriteFileOptions] indicating the character encoding of the data.
  /// The `encoding` option is ignored if [data] is a [Uint8List].
  ///
  /// If a file descriptor is specified as the [file], it will not be closed automatically.
  external void appendFileSync(file, data, [options = BufferEncoding.utf8]);

  /// Changes the permissions of the file system entity specified by [path].
  /// The new file mode is specified by [mode].
  external void chmod(path, int mode, CompletionCallback0 callback);

  /// Synchronously changes the permissions of the file system entity specified by [path].
  /// The new file mode is specified by [mode].
  external void chmodSync(path, int mode);

  /// Changes the ownership of the file system entity specified by [path].
  /// [uid] and [gid] specify the new user and group identity.
  external void chown(path, int uid, int gid, CompletionCallback0 callback);

  /// Synchronously changes the ownership of the file system entity specified by [path].
  /// [uid] and [gid] specify the new user and group identity.
  external void chownSync(path, int uid, int gid);

  /// Closes the file descriptor [fd].
  external void close(int fd, CompletionCallback0 callback);

  /// Synchronously closes the file descriptor [fd].
  external void closeSync(int fd);

  /// Copy the file specified by [src] to [dest]. If [dest] identifies an existing file, that file is replaced.
  ///
  /// `flags` is an optional integer that specifies the behavior of the copy operation.
  /// The only supported flag is `FSConstants.COPYFILE_EXCL`, which causes the copy operation to fail if dest already exists.
  external void copyFile(src, dest, flagsOrCallback, [CompletionCallback0 callback]);

  /// Synchronously copy the file specified by [src] to [dest]. If [dest] identifies an existing file, that file is replaced.
  ///
  /// [flags] is an optional integer that specifies the behavior of the copy operation.
  /// The only supported flag is `FSConstants.COPYFILE_EXCL`, which causes the copy operation to fail if dest already exists.
  external void copyFileSync(src, dest, [int flags = 0]);

  /// Creates a new [Readable] stream for the file specified by [path].
  /// The [options] argument can be a string specifying an encoding or a [CreateStreamOptions] customizing the file reading.
  external FSReadStream<T> createReadStream<T>(path, [CreateStreamOptions options]);

  /// Creates a new [Writable] stream for the file specified by [path].
  /// The [options] argument can be a string specifying an encoding or a [CreateStreamOptions] customizing the file writing.
  external FSWriteStream<T> createWriteStream<T>(path, [CreateStreamOptions options]);

  /// Calls the operating system's [`fdatasync`](http://manpages.ubuntu.com/manpages/xenial/man2/fdatasync.2.html) function on file descriptor [fd].
  external void fdatasync(int fd, CompletionCallback0 callback);

  /// Synchronously calls the operating system's [`fdatasync`](http://manpages.ubuntu.com/manpages/xenial/man2/fdatasync.2.html) function on file descriptor [fd].
  external void fdatasyncSync(int fd);

  /// Checks whether the file system entity with the specified [path] exists.
  /// This throws if any accessibility checks fail, and does nothing otherwise.
  external bool existsSync(path);

  /// Changes the permissions of the file system entity specified by file descriptor [fd].
  /// The new file mode is specified by [mode].
  external void fchmod(int fd, int mode, CompletionCallback0 callback);

  /// Synchronously changes the permissions of the file system entity specified by file descriptor [fd].
  /// The new file mode is specified by [mode].
  external void fchmodSync(int fd, int mode);

  /// Changes the ownership of the file system entity specified by file descriptor [fd].
  /// [uid] and [gid] specify the new user and group identity.
  external void fchown(int fd, int uid, int gid, CompletionCallback0 callback);

  /// Synchronously changes the ownership of the file system entity specified by file descriptor [fd].
  /// [uid] and [gid] specify the new user and group identity.
  external void fchownSync(int fd, int uid, int gid);

  /// Calls the operating system's [`fstat`](http://manpages.ubuntu.com/manpages/xenial/man2/fstat.2.html) function on file descriptor [fd].
  /// The [callback] gets two arguments `(error, stats)` where `stats` is a [Stats] object.
  external void fstat(int fd, CompletionCallback1<Stats> callback);

  /// Synchronously calls the operating system's [`fstat`](http://manpages.ubuntu.com/manpages/xenial/man2/fstat.2.html) function on file descriptor [fd].
  external Stats fstatSync(int fd);

  /// Calls the operating system's [`fsync`](http://manpages.ubuntu.com/manpages/xenial/man2/fsync.2.html) function on file descriptor [fd].
  external void fsync(int fd, CompletionCallback0 callback);

  /// Synchronously calls the operating system's [`fsync`](http://manpages.ubuntu.com/manpages/xenial/man2/fsync.2.html) function on file descriptor [fd].
  external void fsyncSync(int fd);

  /// Truncates the file specified by file descriptor [fd] to the given `length`.
  external void ftruncate(int fd, [lengthOrCallback, CompletionCallback0 callback]);

  /// Synchronously truncates the file specified by file descriptor [fd] to the given [length].
  external void ftruncateSync(int fd, [int length = 0]);

  /// Changes the file system timestamps of the file system entity referenced by file descriptor [fd].
  /// Values can be either integers representing Unix epoch time, [DateTime], or a numeric string like `"123456789.0"`.
  external void futimes(int fd, atime, mtime, CompletionCallback0 callback);

  /// Synchronously changes the file system timestamps of the file system entity referenced by file descriptor [fd].
  /// Values can be either integers representing Unix epoch time, [DateTime], or a numeric string like `"123456789.0"`.
  external void futimesSync(int fd, atime, mtime);

  /// Changes the permissions of the link specified by [path].
  /// The new file mode is specified by [mode].
  external void lchmod(path, int mode, CompletionCallback0 callback);

  /// Synchronously changes the permissions of the link specified by [path].
  /// The new file mode is specified by [mode].
  external void lchmodSync(path, int mode);

  /// Changes the ownership of the link specified by [path].
  /// [uid] and [gid] specify the new user and group identity.
  external void lchown(path, int uid, int gid, CompletionCallback0 callback);

  /// Synchronously changes the ownership of the link specified by [path].
  /// [uid] and [gid] specify the new user and group identity.
  external void lchownSync(path, int uid, int gid);

  /// Creates a hard link at [newPath] that points to [existingPath].
  external void link(existingPath, newPath, CompletionCallback0 callback);

  /// Synchronously creates a hard link at [newPath] that points to [existingPath].
  external void linkSync(existingPath, newPath);

  /// Calls the operating system's [`lstat`](http://manpages.ubuntu.com/manpages/xenial/man2/lstat.2.html) function on [path].
  /// The [callback] gets two arguments `(error, stats)` where `stats` is a [Stats] object.
  external void lstat(path, CompletionCallback1<Stats> callback);

  /// Synchronously calls the operating system's [`lstat`](http://manpages.ubuntu.com/manpages/xenial/man2/lstat.2.html) function on [path].
  external Stats lstatSync(path);

  /// Creates the directory specified by [path].
  external void mkdir(path, modeOrCallback, [CompletionCallback0 callback]);

  /// Synchronously creates the directory specified by [path].
  external void mkdirSync(path, [int mode = 0x1FF]);

  /// Creates a temporary directory.
  ///
  /// Generates six random characters to be appended behind a required [prefix] to create a unique temporary directory.
  /// The created folder path is passed as a string to the [callback]'s second parameter.
  ///
  /// The `options` argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use.
  external void mkdtemp(String prefix, optionsOrCallback, [CompletionCallback1<String> callback]);

  /// Synchronously creates a temporary directory.
  ///
  /// Generates six random characters to be appended behind a required [prefix] to create a unique temporary directory.
  /// Returns the created folder path.
  ///
  /// The [options] argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use.
  external String mkdtempSync(String prefix, [options = BufferEncoding.utf8]);

  /// Opens the file at [path] for random access operations.
  ///
  /// [flags], a string or an integer, indicates the modes in which the file is opened.
  /// `mode` sets the file permissions if the file was created.
  ///
  /// An integer representing the file descriptor is passed to the [callback]'s second parameter.
  external void open(path, flags, modeOrCallback, [CompletionCallback1<int> callback]);

  /// Synchronously opens the file at [path] for random access operations.
  ///
  /// [flags], a string or an integer, indicates the modes in which the file is opened.
  /// [mode] sets the file permissions if the file was created.
  ///
  /// Returns an integer representing the file descriptor.
  external int openSync(path, flags, [int mode = 0x1B6]);

  /// Reads data from the file specified by file descriptor [fd].
  ///
  /// [buffer] is the [Uint8List] that the data will be written to.
  /// [offset] is the offset in the buffer to start writing at.
  /// [length] is an integer specifying the number of bytes to read.
  ///
  /// [position] is an argument specifying where to begin reading from in the file.
  /// If [position] is null, data will be read from the current file position, and the file position will be updated.
  /// If [position] is an integer, the file position will remain unchanged.
  ///
  /// The [callback] is given the three arguments, `(error, bytesRead, buffer)`.
  external void read(int fd, Uint8List buffer, int offset, int length, int position, CompletionCallback2<int, Uint8List> callback);

  /// Lists the sub-directories and files of the directory at [path].
  /// The list of filenames for the directories, files, and links, excluding `.` and `..`, is passed to the [callback]'s second parameter.
  ///
  /// The `options` argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use.
  external void readdir(path, optionsOrCallback, [CompletionCallback1<List<String>> callback]);

  /// Synchronously lists the sub-directories and files of the directory at [path].
  /// The result is a list of filenames for the directories, files, and links, excluding `.` and `..`.
  ///
  /// The [options] argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use.
  external List<String> readdirSync(path, [options = BufferEncoding.utf8]);

  /// Reads the entire contents of the file specified by [path].
  ///
  /// The `options` argument can be a string specifying an encoding or a [ReadFileOptions] indicating the character encoding to use.
  /// If the `encoding` option is specified then the data passed to the [callback] is a string, otherwise it is a [Uint8List].
  external void readFile(path, optionsOrCallback, [CompletionCallback1 callback]);

  /// Synchronously reads the entire contents of the file specified by [path].
  ///
  /// The [options] argument can be a string specifying an encoding or a [ReadFileOptions] indicating the character encoding to use.
  /// If the `encoding` option is specified then this method returns a string, otherwise it returns a [Uint8List].
  external dynamic readFileSync(path, [options]);

  /// Gets the target of the link specified by [path]. Returns the path to the target.
  ///
  /// The `options` argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use for the path
  /// passed to the [callback]. If the encoding is set to [BufferEncoding.buffer], the path returned will be passed as a [Uint8List] object.
  external void readlink(path, optionsOrCallback, [CompletionCallback1 callback]);

  /// Synchronously gets the target of the link specified by [path]. Returns the path to the target.
  ///
  /// The [options] argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use for the returned value.
  /// If the encoding is set to [BufferEncoding.buffer], the path returned will be passed as a [Uint8List] object.
  external dynamic readlinkSync(path, [options = BufferEncoding.utf8]);

  /// Synchronously reads data from the file specified by file descriptor [fd].
  ///
  /// [buffer] is the string or [Uint8List] that the data will be written to.
  /// [offset] is the offset in the buffer to start writing at.
  /// [length] is an integer specifying the number of bytes to read.
  ///
  /// [position] is an argument specifying where to begin reading from in the file.
  /// If [position] is null, data will be read from the current file position, and the file position will be updated.
  /// If [position] is an integer, the file position will remain unchanged.
  ///
  /// Returns the number of bytes read.
  external int readSync(int fd, buffer, int offset, int length, int position);

  /// Resolves the [path] of a file system entity relative to the current working directory,
  /// resolving all symbolic links on the path and resolving all `..` and `.` path segments.
  ///
  /// The `options` argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use for the path
  /// passed to the [callback]. If the encoding is set to [BufferEncoding.buffer], the path returned will be passed as a [Uint8List] object.
  external void realpath(path, optionsOrCallback, [CompletionCallback1 callback]);

  /// Synchronously resolves the [path] of a file system entity relative to the current working directory,
  /// resolving all symbolic links on the path and resolving all `..` and `.` path segments.
  ///
  /// The [options] argument can be a string specifying an encoding or a [EncodingOptions] indicating the character encoding to use for the returned value.
  /// If the encoding is set to [BufferEncoding.buffer], the path returned will be passed as a [Uint8List] object.
  external dynamic realpathSync(path, [options = BufferEncoding.utf8]);

  /// Renames the file system entity located at the specified [oldPath].
  external void rename(String oldPath, String newPath, CompletionCallback0 callback);

  /// Synchronously renames the file system entity located at the specified [oldPath].
  external void renameSync(String oldPath, String newPath);

  /// Deletes the directory specified by [path].
  external void rmdir(path, CompletionCallback0 callback);

  /// Synchronously deletes the directory specified by [path].
  external void rmdirSync(path);

  /// Calls the operating system's [`stat`](http://manpages.ubuntu.com/manpages/xenial/man2/stat.2.html) function on [path].
  /// The [callback] gets two arguments `(error, stats)` where `stats` is a [Stats] object.
  external void stat(path, CompletionCallback1<Stats> callback);

  /// Synchronously calls the operating system's [`stat`](http://manpages.ubuntu.com/manpages/xenial/man2/stat.2.html) function on [path].
  external Stats statSync(path);

  /// Creates a symbolic link at [path] that points to [target].
  /// The `type` argument can be set to `"dir"`, `"file"`, or `"junction"` and is only available on Windows.
  external void symlink(target, path, typeOrCallback, [CompletionCallback0 callback]);

  /// Synchronously creates a symbolic link at [path] that points to [target].
  /// The [type] argument can be set to `"dir"`, `"file"`, or `"junction"` and is only available on Windows.
  external void symlinkSync(target, path, [String type = 'file']);

  /// Truncates the file specified by [path] to the given `length`.
  external void truncate(path, [lengthOrCallback, CompletionCallback0 callback]);

  /// Synchronously truncates the file specified by [path] to the given [length].
  external void truncateSync(path, [int length = 0]);

  /// Deletes the file or link specified by [path].
  external void unlink(path, CompletionCallback0 callback);

  /// Synchronously deletes the file or link specified by [path].
  external void unlinkSync(path);

  /// Stops watching for changes on [filename].
  ///
  /// If [listener] is specified, only that particular listener is removed.
  /// Otherwise, all listeners are removed, effectively stopping watching of [filename].
  external void unwatchFile(filename, [void listener(String eventType, filename)]);

  /// Changes the file system timestamps of the file system entity referenced by [path].
  /// Values can be either integers representing Unix epoch time, [DateTime], or a numeric string like `"123456789.0"`.
  external void utimes(path, atime, mtime, CompletionCallback0 callback);

  /// Synchronously changes the file system timestamps of the file system entity referenced by [path].
  /// Values can be either integers representing Unix epoch time, [DateTime], or a numeric string like `"123456789.0"`.
  external void utimesSync(path, atime, mtime);

  /// Starts watching for changes on the file or directory specified by [path].
  ///
  /// If `options` is provided as a string, it specifies the encoding to be used for the `filename` passed to the [listener].
  /// Otherwise `options` should be passed as a [WatchOptions] object.
  ///
  /// The [listener] callback gets two arguments `(eventType, filename)`.
  /// `eventType` is either `"change"` or `"rename"`, and `filename` is the name of the file which triggered the event.
  external FSWatcher watch(path, [optionsOrListener, void listener(String eventType, filename)]);

  /// Watch for changes on [filename].
  ///
  /// The `options` argument may be omitted. If provided, it should be a [WatchFileOptions] object.
  /// The callback [listener] will be called each time the file is accessed.
  external void watchFile(filename, [optionsOrListener, void listener(Stats current, Stats previous)]);

  /// Writes data to the file specified by file descriptor [fd]. The data can be a string or a [Uint8List].
  /// The callback will be given three arguments `(err, bytesWritten, data)` where `bytesWritten` specifies how many bytes were written from `data`.
  ///
  /// `position` refers to the offset from the beginning of the file where this data should be written.
  /// `encoding` is the expected string encoding.
  /// `offset` determines the part of the buffer to be written, and `length` is an integer specifying the number of bytes to write.
  external void write(int fd, bufferOrString, [int offsetOrPosition, lengthOrEncoding, positionOrCallback, CompletionCallback2<int, dynamic> callback]);

  /// Writes [data] to a [file]. [data] can be a string or a [Uint8List].
  ///
  /// The `options` argument can be a string specifying an encoding or a [WriteFileOptions] indicating the character encoding of the data.
  /// The `encoding` option is ignored if [data] is a [Uint8List].
  ///
  /// If a file descriptor is specified as the [file], it will not be closed automatically.
  external void writeFile(file, data, optionsOrCallback, [CompletionCallback0 callback]);

  /// Synchronously writes [data] to a [file]. [data] can be a string or a [Uint8List].
  ///
  /// The [options] argument can be a string specifying an encoding or a [WriteFileOptions] indicating the character encoding of the data.
  /// The `encoding` option is ignored if [data] is a [Uint8List].
  ///
  /// If a file descriptor is specified as the [file], it will not be closed automatically.
  external void writeFileSync(file, data, [options = BufferEncoding.utf8]);

  /// Synchronously writes data to the file specified by file descriptor [fd]. The data can be a string or a [Uint8List].
  /// Returns the number of bytes written.
  ///
  /// `position` refers to the offset from the beginning of the file where this data should be written.
  /// `encoding` is the expected string encoding.
  /// `offset` determines the part of the buffer to be written, and `length` is an integer specifying the number of bytes to write.
  external int writeSync(int fd, bufferOrString, [int offsetOrPosition, lengthOrEncoding, int position]);
}
