part of nodejs_interop.node;

/// A stream that implements both the [Readable] and [Writable] interfaces.
@JS(r'$stream.Duplex')
class Duplex<T> extends EventEmitter implements Readable<T>, Writable<T> {

  /// Creates a new duplex stream.
  external Duplex([DuplexOptions options]);

  /// The value of [DuplexOptions.readableHighWaterMark] passed when constructing this stream.
  @override
  external int get readableHighWaterMark; // TODO: Node.js 9.3+

  /// The value of [DuplexOptions.writableHighWaterMark] passed when constructing this stream.
  @override
  external int get writableHighWaterMark; // TODO: Node.js 9.3+

  /// Forces all written data to be buffered in memory.
  /// The buffered data will be flushed when either the [uncork] or [end] methods are called.
  @override
  external void cork();

  /// Destroys this stream, and emits the passed [error].
  /// After this call, the stream will release any internal resources.
  @override
  external Duplex<T> destroy([JSError error]);

  /// Signals that no more data will be written to the [Writable].
  ///
  /// The optional [chunk] and `encoding` arguments allow one final additional chunk of data to be written immediately before closing the stream.
  /// For streams not operating in object mode, [chunk] must be a string, `Buffer` or `Uint8List`.
  /// For object mode streams, [chunk] may be any value other than `null`.
  ///
  /// If provided, the optional [callback] function is attached as a listener for the `finish` event.
  @override
  external void end([T chunk, encodingOrCallback, void callback()]);

  /// Returns a value indicating whether this stream is not in flowing mode (i.e. is paused).
  @override
  external bool isPaused();

  /// Causes this stream in flowing mode to stop emitting `data` events, switching out of flowing mode.
  /// Any data that becomes available will remain in the internal buffer.
  @override
  external Duplex<T> pause();

  /// Attaches a [Writable] stream to this [Readable], causing it to switch automatically into flowing mode
  /// and push all of its data to the attached [Writable].
  ///
  /// The flow of data will be automatically managed so that the destination [Writable] stream is not overwhelmed
  /// by a faster [Readable] stream.
  @override
  external Writable<T> pipe(Writable<T> destination, [PipeOptions options]);

  /// Pushes a [chunk] of data to the internal queue for users of the stream to consume.
  /// Returns a value indicating whether additional chunks of data may continued to be pushed.
  ///
  /// Passing [chunk] as `null` signals the end of the stream (EOF), after which no more data can be written.
  @override
  external bool push(T chunk, [String encoding]);

  /// Pulls some data out of the internal buffer and returns it.
  /// If no data available to be read, a `null` reference is returned.
  ///
  /// The optional [size] argument specifies a specific number of bytes to read.
  /// If the [size] argument is not specified, all of the data contained in the internal buffer will be returned.
  @override
  external T read([int size]);

  /// Causes this explicitly paused stream to resume emitting `data` events, switching into flowing mode.
  @override
  external Duplex<T> resume();

  /// Sets the default [encoding] for this [Writable] stream.
  @override
  external Duplex<T> setDefaultEncoding(String encoding);

  /// Sets the character [encoding] for data read from this [Readable] stream.
  @override
  external Duplex<T> setEncoding(String encoding);

  /// Flushes all data buffered since [cork] was called.
  @override
  external void uncork();

  /// Detaches a [Writable] stream previously attached using the [pipe] method.
  /// If the [destination] is not specified, then all pipes are detached.
  @override
  external Duplex<T> unpipe([Writable<T> destination]);

  /// Pushes a [chunk] of data back into the internal buffer.
  /// Returns a value indicating whether additional chunks of data may continued to be pushed.
  @override
  external bool unshift(T chunk);

  /// Wraps an "old style" readable [stream].
  ///
  /// Versions of Node.js prior to v0.10 had streams that did not implement the entire `stream` module API as it is currently defined.
  /// The [wrap] method can be used to create a [Readable] stream that uses the old [stream] as its data source.
  @override
  external void wrap(stream);

  /// Writes some data to the stream, and calls the supplied [callback] once the data has been fully handled.
  ///
  /// If an error occurs, the callback may or may not be called with the error as its first argument.
  /// To reliably detect write errors, add a listener for the `error` event.
  ///
  /// The return value is `true` if the internal buffer is less than the [WritableOptions.highWaterMark] configured
  /// when the stream was created after admitting [chunk]. If `false` is returned, further attempts to write data
  /// to the stream should stop until the `drain` event is emitted.
  @override
  external bool write(T chunk, [encodingOrCallback, CompletionCallback0 callback]);
}

/// Options passed to duplex streams.
@JS()
@anonymous
class DuplexOptions {

  /// Creates a new path object.
  external factory DuplexOptions({
    bool allowHalfOpen = true,
    bool decodeStrings = true,
    void destroy(JSError error, CompletionCallback0 callback),
    String encoding,
    int highWaterMark,
    bool objectMode = false,
    void read(int size),
    int readableHighWaterMark,
    bool readableObjectMode = false,
    int writableHighWaterMark,
    bool writableObjectMode = false,
    void write(chunk, String encoding, CompletionCallback0 callback),
    void writev(List chunks, CompletionCallback0 callback)
  });

  /// Value indicating whether the stream will not automatically end the writable side when the readable side ends.
  external bool get allowHalfOpen;
  external set allowHalfOpen(bool value);

  /// Value indicating whether to decode strings into `Buffer` before passing them to [Duplex.write].
  external bool get decodeStrings;
  external set decodeStrings(bool value);

  /// Implementation for the `_destroy()` method.
  external Function get destroy;
  external set destroy(void fn(JSError error, CompletionCallback0 callback));

  /// The encoding used to decode buffers to strings.
  external String get encoding;
  external set encoding(String value);

  /// The maximum number of bytes to store in the internal buffers.
  external int get highWaterMark;
  external set highWaterMark(int value);

  /// Value indicating whether the stream operates in object mode.
  external bool get objectMode;
  external set objectMode(bool value);

  /// Implementation for the `_read()` method.
  external Function get read;
  external set read(void fn(int size));

  /// The maximum number of bytes the readable side will store in the internal buffer.
  /// Has no effect if [highWaterMark] is provided.
  external int get readableHighWaterMark;
  external set readableHighWaterMark(int value);

  /// Value indicating whether the readable side operates in object mode.
  /// Has no effect if [objectMode] is `true`.
  external bool get readableObjectMode;
  external set readableObjectMode(bool value);

  /// The maximum number of bytes the writable side will store in the internal buffer.
  /// Has no effect if [highWaterMark] is provided.
  external int get writableHighWaterMark;
  external set writableHighWaterMark(int value);

  /// Value indicating whether the writable side operates in object mode.
  /// Has no effect if [objectMode] is `true`.
  external bool get writableObjectMode;
  external set writableObjectMode(bool value);

  /// Implementation for the `_write()` method.
  external Function get write;
  external set write(void fn(chunk, String encoding, CompletionCallback0 callback));

  /// Implementation for the `_writev()` method.
  external Function get writev;
  external set writev(void fn(List chunks, CompletionCallback0 callback));
}
