part of nodejs_interop.node;

/// Options used to customize the way a [Readable] stream is piped into a [Writable] stream.
@JS()
@anonymous
class PipeOptions {

  /// Creates a new options object.
  external factory PipeOptions({bool end = true});

  /// Value indicating whether to end the writer when the reader ends.
  external bool get end;
  external set end(bool value);
}

/// A readable stream is an abstraction for a source from which data is consumed.
@JS(r'$stream.Readable')
class Readable<T> extends EventEmitter {

  /// Creates a new readable stream.
  external Readable([ReadableOptions options]);

  /// The value of [ReadableOptions.highWaterMark] passed when constructing this stream.
  external int get readableHighWaterMark; // TODO: Node.js 9.3+

  /// Destroys this stream, and emits the passed [error].
  /// After this call, the stream will release any internal resources.
  external void destroy([JSError error]);

  /// Returns a value indicating whether this stream is not in flowing mode (i.e. is paused).
  external bool isPaused();

  /// Causes this stream in flowing mode to stop emitting `data` events, switching out of flowing mode.
  /// Any data that becomes available will remain in the internal buffer.
  external Readable<T> pause();

  /// Attaches a [Writable] stream to this [Readable], causing it to switch automatically into flowing mode
  /// and push all of its data to the attached [Writable].
  ///
  /// The flow of data will be automatically managed so that the destination [Writable] stream is not overwhelmed
  /// by a faster [Readable] stream.
  external Writable<T> pipe(Writable<T> destination, [PipeOptions options]);

  /// Pushes a [chunk] of data to the internal queue for users of the stream to consume.
  /// Returns a value indicating whether additional chunks of data may continued to be pushed.
  ///
  /// Passing [chunk] as `null` signals the end of the stream (EOF), after which no more data can be written.
  external bool push(T chunk, [String encoding]);

  /// Pulls some data out of the internal buffer and returns it.
  /// If no data available to be read, a `null` reference is returned.
  ///
  /// The optional [size] argument specifies a specific number of bytes to read.
  /// If the [size] argument is not specified, all of the data contained in the internal buffer will be returned.
  external T read([int size]);

  /// Causes this explicitly paused stream to resume emitting `data` events, switching into flowing mode.
  external Readable<T> resume();

  /// Sets the character [encoding] for data read from this [Readable] stream.
  external Readable<T> setEncoding(String encoding);

  /// Detaches a [Writable] stream previously attached using the [pipe] method.
  /// If the [destination] is not specified, then all pipes are detached.
  external Readable<T> unpipe([Writable<T> destination]);

  /// Pushes a [chunk] of data back into the internal buffer.
  /// Returns a value indicating whether additional chunks of data may continued to be pushed.
  external bool unshift(T chunk);

  /// Wraps an "old style" readable [stream].
  ///
  /// Versions of Node.js prior to v0.10 had streams that did not implement the entire `stream` module API as it is currently defined.
  /// The [wrap] method can be used to create a [Readable] stream that uses the old [stream] as its data source.
  external void wrap(stream);
}

/// Options passed to readable streams.
@JS()
@anonymous
class ReadableOptions {

  /// Creates a new path object.
  external factory ReadableOptions({
    void destroy(JSError error, CompletionCallback0 callback),
    String encoding,
    int highWaterMark,
    bool objectMode = false,
    void read(int size)
  });

  /// Implementation for the `_destroy()` method.
  external Function get destroy;
  external set destroy(void fn(JSError error, CompletionCallback0 callback));

  /// The encoding used to decode buffers to strings.
  external String get encoding;
  external set encoding(String value);

  /// The maximum number of bytes to store in the internal buffer before ceasing to read from the underlying resource.
  /// Defaults to `16384` (16kb), or `16` for streams in object mode.
  external int get highWaterMark;
  external set highWaterMark(int value);

  /// Value indicating whether the stream operates in object mode.
  external bool get objectMode;
  external set objectMode(bool value);

  /// Implementation for the `_read()` method.
  external Function get read;
  external set read(void fn(int size));
}
