import 'package:file/testing.dart';
import 'package:nodejs_interop/io.dart';
import 'package:nodejs_interop/node.dart';
import 'package:path/path.dart' as path;
import 'package:test/test.dart';

/// Tests the features of the [Directory] class.
void main() => group('Directory', () {
  final OSModule os = require('os');

  group('.absolute', () {
    test('should return a `Directory` object with an absolute path', () {
      expect(new Directory('/').absolute, hasPath('/'));
      expect(new Directory('test/fixtures').absolute, hasPath(path.absolute('test/fixtures')));
    });
  });

  group('.current', () {
    test('should return a `Directory` object pointing to the current working directory', () {
      expect(Directory.current, hasPath(process.cwd()));
    });

    test('should allow changing the current working directory', () {
      var cwd = process.cwd();
      Directory.current = dirname;
      expect(Directory.current, hasPath(dirname));
      Directory.current = cwd;
      expect(Directory.current, hasPath(cwd));
    });
  });

  group('.systemTemp', () {
    test('should return a `Directory` object pointing to the system temporary directory', () {
      expect(Directory.systemTemp, hasPath(os.tmpdir()));
    });
  });

  group('.uri', () {
    test('should return a `Uri` with a `file` scheme', () {
      expect(new Directory('/').uri, equals(Uri.parse('file:///')));
      expect(new Directory('/test/fixtures').uri, equals(Uri.parse('file:///test/fixtures/')));
    });
  });

  group('.childDirectory()', () {
    test('should return a new `Directory` inside the directory', () {
      expect(new Directory('test/fixtures').childDirectory('foo.bar'), hasPath('test/fixtures/foo.bar'));
    });
  });

  group('.childFile()', () {
    test('should return a new `File` inside the directory', () {
      expect(new Directory('test/fixtures').childFile('file.txt'), hasPath('test/fixtures/file.txt'));
    });
  });

  group('.childLink()', () {
    test('should return a new `Link` inside the directory', () {
      expect(new Directory('test/fixtures').childLink('link.lnk'), hasPath('test/fixtures/link.lnk'));
    });
  });

  group('.create()', () {
    test('should recursively create the specified directory', () async {
      var directory = new Directory('var/test/file/directory.create');
      expect(directory, isNot(exists));

      await directory.create(recursive: true);
      expect(directory, exists);

      var child = directory.childDirectory('child');
      await child.create();
      expect(child, exists);
    });
  });

  group('.createSync()', () {
    test('should recursively create the specified directory', () {
      var directory = new Directory('var/test/file/directory.createSync');
      expect(directory, isNot(exists));

      directory.createSync(recursive: true);
      expect(directory, exists);
      expect(directory.childDirectory('child')..createSync(), exists);
    });
  });

  /*
  group('.createTemp()', () {
    test('should TODO', () async {
      var directory = new Directory('var/test/file/directory.createSync');
      var tempDir = await directory.createTemp('foo');
      expect(tempDir, exists);
      expect(tempDir.path, startsWith(platform.isWindows ? r'C:\Windows\Temp\foo' : '/tmp/foo'));
    });
  });

  group('.createTempSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.delete()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.deleteSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.list()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.listSync()', () {
    test('should TODO', () {
      // TODO
    });
  });

  group('.rename()', () {
    test('should TODO', () async {
      // TODO
    });
  });

  group('.renameSync()', () {
    test('should TODO', () {
      // TODO
    });
  });
  */
});
