// ignore_for_file: avoid_slow_async_io
import 'package:file/testing.dart';
import 'package:nodejs_interop/io.dart';
import 'package:path/path.dart' as path;
import 'package:test/test.dart';

/// Tests the features of the [FileSystemEntity] class.
void main() => group('FileSystemEntity', () {
  const platform = const Platform();
  final matchesPerms = anyOf(equals(int.parse('664', radix: 8)), equals(int.parse('644', radix: 8)));

  group('.basename', () {
    test('should return the part of path after the last separator', () {
      expect(new Directory('test/fixtures').basename, equals('fixtures'));
      expect(new File('test/fixtures/file.txt').basename, equals('file.txt'));
      expect(new Link('test/fixtures/link.lnk').basename, equals('link.lnk'));
    });
  });

  group('.dirname', () {
    test('should return the part of path before the last separator', () {
      expect(new Directory('test/fixtures').dirname, equals('test'));
      expect(new File('test/fixtures/file.txt').dirname, equals(platform.isWindows ? r'test\fixtures' : 'test/fixtures'));
      expect(new Link('test/fixtures/link.lnk').dirname, equals(platform.isWindows ? r'test\fixtures' : 'test/fixtures'));
    });
  });

  group('.isAbsolute', () {
    test('should return `true` if the path is absolute', () {
      expect(new Directory('test/fixtures').isAbsolute, isFalse);
      expect(new Directory('/var/log').isAbsolute, isTrue);

      expect(new File('test/fixtures/file.txt').isAbsolute, isFalse);
      expect(new File('/var/log/syslog').isAbsolute, isTrue);

      expect(new Link('test/fixtures/link.lnk').isAbsolute, isFalse);
      expect(new Link('/var/log/syslog').isAbsolute, isTrue);
    });
  });

  group('.parent', () {
    test('should return the parent entity', () {
      expect(new Directory('/').parent, hasPath('/'));
      expect(new Directory('test/fixtures').parent, hasPath('test'));
      expect(new File('test/fixtures/file.txt').parent, hasPath('test/fixtures'));
      expect(new Link('test/fixtures/link.lnk').parent, hasPath('test/fixtures'));
    });
  });

  group('.path', () {
    test("should return the entity's path", () {
      expect(new Directory('/'), hasPath('/'));
      expect(new Directory('test/fixtures'), hasPath('test/fixtures'));
      expect(new File('test/fixtures/file.txt'), hasPath('test/fixtures/file.txt'));
      expect(new Link('test/fixtures/link.lnk'), hasPath('test/fixtures/link.lnk'));
    });
  });

  group('.isWatchSupported', () {
    test('should return `true` if `watch()` is supported', () {
      expect(FileSystemEntity.isWatchSupported, !platform.isMacOS);
    });
  });

  group('.exists()', () {
    test('should return `true` if the specified entity is a directory', () async {
      expect(await new Directory('test/fixtures').exists(), isTrue);
      expect(await new File('test/fixtures/file.txt').exists(), isTrue);
      expect(await new Link('test/fixtures/link.lnk').exists(), isTrue);
      expect(await new File('not_found.dat').exists(), isFalse);
    });
  });

  group('.existsSync()', () {
    test('should return `true` if the specified entity is a directory', () {
      expect(new Directory('test/fixtures').existsSync(), isTrue);
      expect(new File('test/fixtures/file.txt').existsSync(), isTrue);
      expect(new Link('test/fixtures/link.lnk').existsSync(), isTrue);
      expect(new File('not_found.dat').existsSync(), isFalse);
    });
  });

  group('.identical()', () {
    test('should return `true` if the two paths refer to the same entity', () async {
      expect(await FileSystemEntity.identical('test/fixtures/link.lnk', 'test/fixtures/file.txt'), isFalse);
      expect(await FileSystemEntity.identical('test/fixtures/file.txt', 'test/fixtures/file.txt'), isTrue);
    });

    test('should throw an exception if a path does not exist', () {
      expect(FileSystemEntity.identical('not_found.dat', 'test/fixtures/file.txt'), throwsFileSystemException());
    });
  });

  group('.identicalSync()', () {
    test('should return `true` if the two paths refer to the same entity', () {
      expect(FileSystemEntity.identicalSync('test/fixtures/link.lnk', 'test/fixtures/file.txt'), isFalse);
      expect(FileSystemEntity.identicalSync('test/fixtures/file.txt', 'test/fixtures/file.txt'), isTrue);
    });

    test('should throw an exception if a path does not exist', () {
      expect(() => FileSystemEntity.identicalSync('not_found.dat', 'test/fixtures/file.txt'), throwsFileSystemException());
    });
  });

  group('.isDirectory()', () {
    test('should return `true` if the specified entity is a directory', () async {
      expect(await FileSystemEntity.isDirectory('test/fixtures'), isTrue);
      expect(await FileSystemEntity.isDirectory('test/fixtures/file.txt'), isFalse);
      expect(await FileSystemEntity.isDirectory('test/fixtures/link.lnk'), isFalse);
    });
  });

  group('.isDirectorySync()', () {
    test('should return `true` if the specified entity is a directory', () {
      expect(FileSystemEntity.isDirectorySync('test/fixtures'), isTrue);
      expect(FileSystemEntity.isDirectorySync('test/fixtures/file.txt'), isFalse);
      expect(FileSystemEntity.isDirectorySync('test/fixtures/link.lnk'), isFalse);
    });
  });

  group('.isFile()', () {
    test('should return `true` if the specified entity is a file', () async {
      expect(await FileSystemEntity.isFile('test/fixtures'), isFalse);
      expect(await FileSystemEntity.isFile('test/fixtures/file.txt'), isTrue);
      expect(await FileSystemEntity.isFile('test/fixtures/link.lnk'), isFalse);
    });
  });

  group('.isFileSync()', () {
    test('should return `true` if the specified entity is a file', () {
      expect(FileSystemEntity.isFileSync('test/fixtures'), isFalse);
      expect(FileSystemEntity.isFileSync('test/fixtures/file.txt'), isTrue);
      expect(FileSystemEntity.isFileSync('test/fixtures/link.lnk'), isFalse);
    });
  });

  group('.isLink()', () {
    test('should return `true` if the specified entity is a link', () async {
      expect(await FileSystemEntity.isLink('test/fixtures'), isFalse);
      expect(await FileSystemEntity.isLink('test/fixtures/file.txt'), isFalse);
      expect(await FileSystemEntity.isLink('test/fixtures/link.lnk'), isTrue);
    });
  });

  group('.isLinkSync()', () {
    test('should return `true` if the specified entity is a link', () {
      expect(FileSystemEntity.isLinkSync('test/fixtures'), isFalse);
      expect(FileSystemEntity.isLinkSync('test/fixtures/file.txt'), isFalse);
      expect(FileSystemEntity.isLinkSync('test/fixtures/link.lnk'), isTrue);
    });
  });

  group('.parentOf()', () {
    test('should return the part of path before the last separator', () {
      expect(FileSystemEntity.parentOf('/'), equals('/'));
      expect(FileSystemEntity.parentOf('test/fixtures'), equals('test'));
      expect(FileSystemEntity.parentOf('test/fixtures/file.txt'), equals(platform.isWindows ? r'test\fixtures' : 'test/fixtures'));
    });
  });

  group('.resolveSymbolicLinks()', () {
    test('should return a `FileStat` instance for an existing entity', () async {
      var realpath = await new File('test/fixtures/link.lnk').resolveSymbolicLinks();
      expect(realpath, equals(path.absolute('test/fixtures/file.txt')));
    });

    test('should throw an exception if the entity does not exist', () {
      var file = new File('not_found.dat');
      expect(file.resolveSymbolicLinks(), throwsFileSystemException());
    });
  });

  group('.resolveSymbolicLinksSync()', () {
    test('should return a `FileStat` instance for an existing entity', () {
      var realpath = new File('test/fixtures/link.lnk').resolveSymbolicLinksSync();
      expect(realpath, equals(path.absolute('test/fixtures/file.txt')));
    });

    test('should throw an exception if the entity does not exist', () {
      var file = new File('not_found.dat');
      expect(() => file.resolveSymbolicLinksSync(), throwsFileSystemException());
    });
  });

  group('.stat()', () {
    test('should return a `FileStat` instance for an existing entity', () async {
      var stat = await new File('test/fixtures/file.txt').stat();
      expect(stat.mode & 0xFFF, matchesPerms);
      expect(stat.size, equals(0));
      expect(stat.type, equals(FileSystemEntityType.FILE));
    });

    test('should return an instance with a `FileSystemEntityType.NOT_FOUND` type if an error occurred', () async {
      var stat = await new File('not_found.dat').stat();
      expect(stat.type, equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.statSync()', () {
    test('should return a `FileStat` instance for an existing entity', () {
      var stat = new File('test/fixtures/file.txt').statSync();
      expect(stat.mode & 0xFFF, matchesPerms);
      expect(stat.size, equals(0));
      expect(stat.type, equals(FileSystemEntityType.FILE));
    });

    test('should return an instance with a `FileSystemEntityType.NOT_FOUND` type if an error occurred', () {
      var stat = new File('not_found.dat').statSync();
      expect(stat.type, equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.type()', () {
    test('should return the `FileSystemEntityType` of an existing entity', () async {
      expect(await FileSystemEntity.type('test/fixtures'), equals(FileSystemEntityType.DIRECTORY));
      expect(await FileSystemEntity.type('test/fixtures/file.txt'), equals(FileSystemEntityType.FILE));
      expect(await FileSystemEntity.type('test/fixtures/link.lnk'), equals(FileSystemEntityType.LINK));
    });

    test('should return `FileSystemEntityType.NOT_FOUND` if an error occurred', () async {
      expect(await FileSystemEntity.type('not_found.dat'), equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.typeSync()', () {
    test('should return the `FileSystemEntityType` of an existing entity', () {
      expect(FileSystemEntity.typeSync('test/fixtures'), equals(FileSystemEntityType.DIRECTORY));
      expect(FileSystemEntity.typeSync('test/fixtures/file.txt'), equals(FileSystemEntityType.FILE));
      expect(FileSystemEntity.typeSync('test/fixtures/link.lnk'), equals(FileSystemEntityType.LINK));
    });

    test('should return `FileSystemEntityType.NOT_FOUND` if an error occurred', () {
      expect(FileSystemEntity.typeSync('not_found.dat'), equals(FileSystemEntityType.NOT_FOUND));
    });
  });
});
