import 'package:file/testing.dart';
import 'package:nodejs_interop/io.dart';
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [FileSystem] class.
void main() => group('FileSystem', () {
  const platform = const Platform();
  final matchesPerms = anyOf(equals(int.parse('664', radix: 8)), equals(int.parse('644', radix: 8)));
  final OSModule os = require('os');

  group('.currentDirectory', () {
    test('should return a `Directory` object pointing to the current working directory', () {
      expect(const FileSystem().currentDirectory, hasPath(process.cwd()));
    });

    test('should allow changing the current working directory', () {
      var cwd = process.cwd();
      var fs = const FileSystem()..currentDirectory = dirname;
      expect(fs.currentDirectory, hasPath(dirname));
      fs.currentDirectory = cwd;
      expect(fs.currentDirectory, hasPath(cwd));
    });
  });

  group('.isWatchSupported', () {
    test('should return `true` if `FileSystemEntity.watch()` is supported', () {
      expect(const FileSystem().isWatchSupported, !platform.isMacOS);
    });
  });

  group('.path', () {
    test('should return an object for manipulating paths', () {
      expect(const FileSystem().path.style.name, equals(platform.isWindows ? 'windows' : 'posix'));
    });
  });

  group('.systemTempDirectory', () {
    test('should return a `Directory` object pointing to the system temporary directory', () {
      expect(const FileSystem().systemTempDirectory, hasPath(os.tmpdir()));
    });
  });

  group('.identical()', () {
    test('should return `true` if the two paths refer to the same entity', () async {
      var fs = const FileSystem();
      expect(await fs.identical('test/fixtures/link.lnk', 'test/fixtures/file.txt'), isFalse);
      expect(await fs.identical('test/fixtures/file.txt', 'test/fixtures/file.txt'), isTrue);
    });

    test('should throw an exception if a path does not exist', () {
      expect(const FileSystem().identical('not_found.dat', 'test/fixtures/file.txt'), throwsFileSystemException());
    });
  });

  group('.identicalSync()', () {
    test('should return `true` if the two paths refer to the same entity', () {
      var fs = const FileSystem();
      expect(fs.identicalSync('test/fixtures/link.lnk', 'test/fixtures/file.txt'), isFalse);
      expect(fs.identicalSync('test/fixtures/file.txt', 'test/fixtures/file.txt'), isTrue);
    });

    test('should throw an exception if a path does not exist', () {
      expect(() => const FileSystem().identicalSync('not_found.dat', 'test/fixtures/file.txt'), throwsFileSystemException());
    });
  });

  group('.stat()', () {
    test('should return a `FileStat` instance for an existing entity', () async {
      var stat = await const FileSystem().stat('test/fixtures/file.txt');
      expect(stat.mode & 0xFFF, matchesPerms);
      expect(stat.size, equals(0));
      expect(stat.type, equals(FileSystemEntityType.FILE));
    });

    test('should return an instance with a `FileSystemEntityType.NOT_FOUND` type if an error occurred', () async {
      var stat = await const FileSystem().stat('not_found.dat');
      expect(stat.type, equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.statSync()', () {
    test('should return a `FileStat` instance for an existing entity', () {
      var stat = const FileSystem().statSync('test/fixtures/file.txt');
      expect(stat.mode & 0xFFF, matchesPerms);
      expect(stat.size, equals(0));
      expect(stat.type, equals(FileSystemEntityType.FILE));
    });

    test('should return an instance with a `FileSystemEntityType.NOT_FOUND` type if an error occurred', () {
      var stat = const FileSystem().statSync('not_found.dat');
      expect(stat.type, equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.type()', () {
    test('should return the `FileSystemEntityType` of an existing entity', () async {
      var fs = const FileSystem();
      expect(await fs.type('test/fixtures'), equals(FileSystemEntityType.DIRECTORY));
      expect(await fs.type('test/fixtures/file.txt'), equals(FileSystemEntityType.FILE));
      expect(await fs.type('test/fixtures/link.lnk'), equals(FileSystemEntityType.LINK));
    });

    test('should return `FileSystemEntityType.NOT_FOUND` if an error occurred', () async {
      expect(await const FileSystem().type('not_found.dat'), equals(FileSystemEntityType.NOT_FOUND));
    });
  });

  group('.typeSync()', () {
    test('should return the `FileSystemEntityType` of an existing entity', () {
      var fs = const FileSystem();
      expect(fs.typeSync('test/fixtures'), equals(FileSystemEntityType.DIRECTORY));
      expect(fs.typeSync('test/fixtures/file.txt'), equals(FileSystemEntityType.FILE));
      expect(fs.typeSync('test/fixtures/link.lnk'), equals(FileSystemEntityType.LINK));
    });

    test('should return `FileSystemEntityType.NOT_FOUND` if an error occurred', () {
      expect(const FileSystem().typeSync('not_found.dat'), equals(FileSystemEntityType.NOT_FOUND));
    });
  });
});
