import 'package:nodejs_interop/io.dart';
import 'package:nodejs_interop/js.dart';
import 'package:nodejs_interop/node.dart';
import 'package:path/path.dart' as path;
import 'package:test/test.dart';

/// Tests the features of the [Platform] class.
void main() => group('Platform', () {
  final isMacOS = process.platform == 'darwin';
  final isWindows = process.platform == 'win32';

  group('.environment', () {
    var env = const Platform().environment;

    test('should reflect the values of the `process.env` property', () {
      var jsEnv = getProperties(process.env);
      expect(env.length, equals(jsEnv.length));
      for (var name in env.keys) expect(env[name], equals(jsEnv[name]), reason: name);
    });

    test('should be an unmodifiable map', () {
      expect(() => env['Foo'] = 'Bar', throwsUnsupportedError);
    });

    test('should be case-insensitive on Windows, case-sensitive on other platforms', () {
      expect(env['home'], isWindows ? isNotEmpty : isNull);
    });
  });

  group('.executable', () {
    test('should be `node.exe` on Windows and `node` on other platforms', () {
      expect(const Platform().executable, equals(isWindows ? 'node.exe' : 'node'));
    });
  });

  group('.localeName', () {
    test('should return a locale formatted like "<language>_<REGION>"', () {
      expect(const Platform().localeName, matches(new RegExp(r'^[a-z]{2,3}_[A-Z]{2,3}$')));
    });
  });

  group('.operatingSystem', () {
    test('should be `macos` on macOS, `windows` on Windows and `linux` on other platforms', () {
      expect(const Platform().operatingSystem, equals(isWindows ? 'windows' : (isMacOS ? 'macos' : 'linux')));
    });
  });

  group('.pathSeparator', () {
    test('should be `\\` on Windows and `/` on other platforms', () {
      expect(const Platform().pathSeparator, equals(isWindows ? r'\' : '/'));
    });
  });

  group('.resolvedExecutable', () {
    test('should be the absolute path to `node.exe` on Windows and `node` on other platforms', () {
      var resolvedExecutable = const Platform().resolvedExecutable;
      expect(resolvedExecutable, endsWith(isWindows ? r'\node.exe' : '/node'));
      expect(path.isAbsolute(resolvedExecutable), isTrue);
    });
  });

  group('.version', () {
    test('should be a SemVer string representing the `process.version` property', () {
      expect(const Platform().version, matches(new RegExp(r'^\d+(\.\d+){2}$')));
    });
  });
});
