import 'package:nodejs_interop/io.dart';
import 'package:test/test.dart';

/// Tests the features of the [InternetAddress] class.
void main() => group('InternetAddress', () {
  var ipv4Address = new InternetAddress('163.172.221.182');
  var ipv6Address = new InternetAddress('2a00:1450:4007:811::200e');

  group('.address', () {
    test('should return "0.0.0.0" for the IPv4 any address', () {
      expect(InternetAddress.ANY_IP_V4.address, equals('0.0.0.0'));
    });

    test('should return "::" for the IPv6 any address', () {
      expect(InternetAddress.ANY_IP_V6.address, equals('::'));
    });

    test('should return "127.0.0.1" for the IPv4 loopback address', () {
      expect(InternetAddress.LOOPBACK_IP_V4.address, equals('127.0.0.1'));
    });

    test('should return "::1" for the IPv6 loopback address', () {
      expect(InternetAddress.LOOPBACK_IP_V6.address, equals('::1'));
    });

    test('should return the address passed to the constructor', () {
      expect(ipv4Address.address, equals('163.172.221.182'));
      expect(ipv6Address.address, equals('2a00:1450:4007:811::200e'));
    });
  });

  group('.host', () {
    test('should return the `address` by default', () {
      expect(InternetAddress.ANY_IP_V4.host, equals(InternetAddress.ANY_IP_V4.address));
      expect(InternetAddress.ANY_IP_V6.host, equals(InternetAddress.ANY_IP_V6.address));
      expect(InternetAddress.LOOPBACK_IP_V4.host, equals(InternetAddress.LOOPBACK_IP_V4.address));
      expect(InternetAddress.LOOPBACK_IP_V6.host, equals(InternetAddress.LOOPBACK_IP_V6.address));
      expect(ipv4Address.host, equals(ipv4Address.address));
      expect(ipv6Address.host, equals(ipv6Address.address));
    });
  });

  group('.isLinkLocal', () {
    test('should return `true` if the internet address is a link-local one, otherwise `false`', () {
      expect(InternetAddress.ANY_IP_V4.isLinkLocal, isFalse);
      expect(InternetAddress.ANY_IP_V6.isLinkLocal, isFalse);
      expect(InternetAddress.LOOPBACK_IP_V4.isLinkLocal, isFalse);
      expect(InternetAddress.LOOPBACK_IP_V6.isLinkLocal, isFalse);
      expect(ipv4Address.isLinkLocal, isFalse);
      expect(ipv6Address.isLinkLocal, isFalse);
      expect(new InternetAddress('169.254.0.1').isLinkLocal, isTrue);
      expect(new InternetAddress('fe80::1').isLinkLocal, isTrue);
    });
  });

  group('.isLoopback', () {
    test('should return `true` if the internet address is a loopback one, otherwise `false`', () {
      expect(InternetAddress.ANY_IP_V4.isLoopback, isFalse);
      expect(InternetAddress.ANY_IP_V6.isLoopback, isFalse);
      expect(InternetAddress.LOOPBACK_IP_V4.isLoopback, isTrue);
      expect(InternetAddress.LOOPBACK_IP_V6.isLoopback, isTrue);
      expect(ipv4Address.isLoopback, isFalse);
      expect(ipv6Address.isLoopback, isFalse);
    });
  });

  group('.isMulticast', () {
    test('should return `true` if the internet address is a multicast one, otherwise `false`', () {
      expect(InternetAddress.ANY_IP_V4.isMulticast, isFalse);
      expect(InternetAddress.ANY_IP_V6.isMulticast, isFalse);
      expect(InternetAddress.LOOPBACK_IP_V4.isMulticast, isFalse);
      expect(InternetAddress.LOOPBACK_IP_V6.isMulticast, isFalse);
      expect(ipv4Address.isMulticast, isFalse);
      expect(ipv6Address.isMulticast, isFalse);
      expect(new InternetAddress('224.0.0.1').isMulticast, isTrue);
      expect(new InternetAddress('ff01::1').isMulticast, isTrue);
    });
  });

  group('.rawAddress', () {
    test('should return a 4 or 16 byte long list', () {
      expect(InternetAddress.ANY_IP_V4.rawAddress, orderedEquals(new List<int>.filled(4, 0)));
      expect(InternetAddress.ANY_IP_V6.rawAddress, orderedEquals(new List<int>.filled(16, 0)));
      expect(InternetAddress.LOOPBACK_IP_V4.rawAddress, orderedEquals(<int>[127, 0, 0, 1]));
      expect(InternetAddress.LOOPBACK_IP_V6.rawAddress, orderedEquals(<int>[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]));
      expect(ipv4Address.rawAddress, orderedEquals(<int>[163, 172, 221, 182]));
      expect(ipv6Address.rawAddress, orderedEquals(<int>[42, 0, 20, 80, 64, 7, 8, 17, 0, 0, 0, 0, 0, 0, 32, 14]));
    });
  });

  group('.type', () {
    test('should return the type of the internet address', () {
      expect(InternetAddress.ANY_IP_V4.type, equals(InternetAddressType.IP_V4));
      expect(InternetAddress.ANY_IP_V6.type, equals(InternetAddressType.IP_V6));
      expect(InternetAddress.LOOPBACK_IP_V4.type, equals(InternetAddressType.IP_V4));
      expect(InternetAddress.LOOPBACK_IP_V6.type, equals(InternetAddressType.IP_V6));
      expect(ipv4Address.type, equals(InternetAddressType.IP_V4));
      expect(ipv6Address.type, equals(InternetAddressType.IP_V6));
    });
  });

  group('.lookup()', () {
    test('should return the internet address corresponding to the hostname', () async {
      var address = (await InternetAddress.lookup('localhost')).single;
      expect(address.address, equals(InternetAddress.LOOPBACK_IP_V4.address));
      expect(address.host, equals('localhost'));

      address = (await InternetAddress.lookup('ip6-localhost')).single;
      expect(address.address, equals(InternetAddress.LOOPBACK_IP_V6.address));
      expect(address.host, equals('ip6-localhost'));

      address = (await InternetAddress.lookup('belin.io')).single;
      expect(address.address, equals(ipv4Address.address));
      expect(address.host, equals('belin.io'));

      var addresses = await InternetAddress.lookup('google.com');
      expect(addresses, isNotEmpty);
      expect(addresses.every((address) => address.host == 'google.com'), isTrue);
    });
  });

  group('.reverse()', () {
    test('should perform a reverse DNS lookup on the specified address', () async {
      var address = await ipv4Address.reverse();
      expect(address.address, equals(ipv4Address.address));
      expect(address.host, anyOf(equals('ns.belin.io'), equals('sd-100506'), equals('sd-100506.dedibox.fr')));

      address = await ipv6Address.reverse();
      expect(address.address, equals(ipv6Address.address));
      expect(address.host, equals('par21s04-in-x0e.1e100.net'));
    });
  });
});
