import 'dart:async';
import 'package:nodejs_interop/js.dart';
import 'package:test/test.dart';

/// Tests the features of the `nodejs_interop.js` functions.
void main() {
  group('asFuture()', () {
    test('should return a future that completes with a value when the promise resolves', () async {
      expect(await asFuture<String>(Promise.resolve('foo')), equals('foo'));
    });

    test('should return a future that completes with an error when the promise rejects', () {
      expect(asFuture<String>(Promise.reject(new Exception())), throwsException);
    });
  });

  group('asPromise()', () {
    test('should return a promise that resolves when the future completes with a value', () async {
      var promise = asPromise<String>(new Future<String>.value('foo'));
      expect(await asFuture<String>(promise), equals('foo'));
    });

    test('should return a promise that rejects when the future completes with an error', () {
      var promise = asPromise(new Future.error(new Exception()));
      expect(asFuture(promise), throwsException);
    });
  });

  group('dartify()', () {
    test('should leave unchanged the supported Dart types', () {
      var now = new DateTime.now();
      expect(dartify(null), isNull);
      expect(dartify(false), isFalse);
      expect(dartify(123), equals(123));
      expect(dartify(-123.456), equals(-123.456));
      expect(dartify(now), equals(now));
      expect(dartify('foo'), equals('foo'));
    });

    test('should convert JavaScript arrays to `List`', () {
      List array = jsify(['foo', 'bar']);
      expect(dartify(array), orderedEquals(<String>['foo', 'bar']));
    });

    test('should convert JavaScript objects to `Map`', () {
      Object object = jsify({'foo': 'bar'});
      expect(dartify(object), allOf(isMap, hasLength(1), containsPair('foo', 'bar')));
    });
  });

  group('deleteProperty()', () {
    test('should remove the property from the specified object', () {
      Object object = jsify({'foo': 'bar'});
      expect(hasProperty(object, 'foo'), isTrue);
      expect(deleteProperty(object, 'foo'), isTrue);
      expect(hasProperty(object, 'foo'), isFalse);
    });
  });

  group('getProperties()', () {
    test('should return the enumerable properties of the specified object', () {
      Object object = jsify({'foo': 'bar'});
      expect(getProperties(object), allOf(hasLength(1), containsPair('foo', 'bar')));

      setProperty(object, 'baz', 'qux');
      expect(getProperties(object), allOf(hasLength(2), containsPair('baz', 'qux')));
    });
  });
}
