import 'dart:js';
import 'package:nodejs_interop/js.dart';
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [AssertModule] class.
void main() => group('AssertModule', () {
  final AssertModule _assert = require('assert');

  group('.deepEqual()', () {
    test('should not throw an assertion error if the actual and expected value are deeply equal', () {
      Object actual = jsify({'foo': 123});
      Object expected = jsify({'foo': '123'});
      expect(() => _assert.deepEqual(actual, expected), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not deeply equal', () {
      Object actual = jsify({'foo': 'bar'});
      Object expected = jsify({'foo': 'qux'});
      expect(() => _assert.deepEqual(actual, expected), throwsException);
    });
  });

  group('.deepStrictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are deeply and strictly equal', () {
      Object actual = jsify({'foo': '123'});
      Object expected = jsify({'foo': '123'});
      expect(() => _assert.deepStrictEqual(actual, expected), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not deeply and strictly equal', () {
      Object actual = jsify({'foo': '123'});
      Object expected = jsify({'foo': 123});
      expect(() => _assert.deepStrictEqual(actual, expected), throwsException);
    });
  });

  group('.doesNotThrow()', () {
    test('should not throw an assertion error if the block does not throw an error', () {
      var block = allowInterop(() => null);
      expect(() => _assert.doesNotThrow(block), returnsNormally);
    });

    test('should throw an assertion error if the block throws an error', () {
      var block = allowInterop(() => throw new Exception());
      expect(() => _assert.doesNotThrow(block), throwsException);
    });
  });

  group('.equal()', () {
    test('should not throw an assertion error if the actual and expected value are equal', () {
      expect(() => _assert.equal(123, '123'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not equal', () {
      expect(() => _assert.equal('foo', 'bar'), throwsException);
    });
  });

  group('.fail()', () {
    test('should throw an assertion error', () {
      expect(_assert.fail, throwsException);
    });
  });

  group('.ifError()', () {
    test('should not throw an assertion error if the value is falsy', () {
      expect(() => _assert.ifError(null), returnsNormally);
    });

    test('should throw an assertion error if the value is truthy', () {
      expect(() => _assert.ifError(new Exception()), throwsException);
    });
  });

  group('.notDeepEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not deeply equal', () {
      Object actual = jsify({'foo': 'bar'});
      Object expected = jsify({'foo': 'qux'});
      expect(() => _assert.notDeepEqual(actual, expected), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are deeply equal', () {
      Object actual = jsify({'foo': 123});
      Object expected = jsify({'foo': '123'});
      expect(() => _assert.notDeepEqual(actual, expected), throwsException);
    });
  });

  group('.notDeepStrictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not deeply and strictly equal', () {
      Object actual = jsify({'foo': '123'});
      Object expected = jsify({'foo': 123});
      expect(() => _assert.notDeepStrictEqual(actual, expected), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are deeply and strictly equal', () {
      Object actual = jsify({'foo': '123'});
      Object expected = jsify({'foo': '123'});
      expect(() => _assert.notDeepStrictEqual(actual, expected), throwsException);
    });
  });

  group('.notEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not equal', () {
      expect(() => _assert.notEqual('foo', 'bar'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are equal', () {
      expect(() => _assert.notEqual(123, '123'), throwsException);
    });
  });

  group('.notStrictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are not strictly equal', () {
      expect(() => _assert.notStrictEqual(123, '123'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are strictly equal', () {
      expect(() => _assert.notStrictEqual('123', '123'), throwsException);
    });
  });

  group('.ok()', () {
    test('should not throw an assertion error if the value is truthy', () {
      expect(() => _assert(true), returnsNormally);
      expect(() => _assert.ok(true), returnsNormally);
    });

    test('should throw an assertion error if the value is falsy', () {
      expect(() => _assert(false), throwsException);
      expect(() => _assert.ok(false), throwsException);
    });
  });

  group('.strictEqual()', () {
    test('should not throw an assertion error if the actual and expected value are strictly equal', () {
      expect(() => _assert.strictEqual('123', '123'), returnsNormally);
    });

    test('should throw an assertion error if the actual and expected value are not strictly equal', () {
      expect(() => _assert.strictEqual(123, '123'), throwsException);
    });
  });

  group('.throws()', () {
    test('should not throw an assertion error if the block throws an error', () {
      var block = allowInterop(() => throw new Exception());
      expect(() => _assert.throws(block), returnsNormally);
    });

    test('should throw an assertion error if the block does not throw an error', () {
      var block = allowInterop(() => null);
      expect(() => _assert.throws(block), throwsException);
    });
  });
});
