// ignore_for_file: unnecessary_lambdas
import 'package:nodejs_interop/node.dart';
import 'package:test/test.dart';

/// Tests the features of the [Resolver] class.
void main() => group('Resolver', () {
  loadLibrary<DnsModule>('dns');
  final NetModule net = require('net');

  group('.getServers()/.setServers()', () {
    test('should return a non-empty list', () {
      expect(new Resolver().getServers(), isNotEmpty);
    });

    test('should get and set the list of DNS servers', () {
      var servers = const ['127.0.0.1', '::1'];
      var resolver = new Resolver()..setServers(servers);
      expect(resolver.getServers(), orderedEquals(servers));
    });
  });

  group('.resolve4()', () {
    test('should resolve the hostname into a list of IPv4 addresses', () async {
      var addresses = await fn1cb1ToFuture<List>((host, cb) => new Resolver().resolve4(host, cb), 'belin.io');
      expect(addresses, hasLength(1));
      expect(net.isIPv4(addresses.first), isTrue);
    });

    test('should resolve the hostname into a list of IPv4 `AddressRecord` instances', () async {
      var addresses = await fn2cb1ToFuture<List>((host, opts, cb) => new Resolver().resolve4(host, opts, cb), 'belin.io', new ResolveOptions(ttl: true));
      expect(addresses, hasLength(1));

      AddressRecord record = addresses.first;
      expect(net.isIPv4(record.address), isTrue);
    });
  });

  group('.resolve6()', () {
    test('should resolve the hostname into a list of IPv6 addresses', () async {
      var addresses = await fn1cb1ToFuture<List>((host, cb) => new Resolver().resolve6(host, cb), 'google.com');
      expect(addresses, hasLength(1));
      expect(net.isIPv6(addresses.first), isTrue);
    });

    test('should resolve the hostname into a list of IPv6 `AddressRecord` instances', () async {
      var addresses = await fn2cb1ToFuture<List>((host, opts, cb) => new Resolver().resolve6(host, opts, cb), 'google.com', new ResolveOptions(ttl: true));
      expect(addresses, hasLength(1));

      AddressRecord record = addresses.first;
      expect(net.isIPv6(record.address), isTrue);
    });
  });

  group('.resolveAny()', () {
    test('should resolve the hostname into a list of `DnsRecord` instances', () async {
      expect(await fn1cb1ToFuture<List<DnsRecord>>((host, cb) => new Resolver().resolveAny(host, cb), 'belin.io'), isNotEmpty);
    });
  });

  group('.resolveCname()', () {
    test('should resolve the hostname into a list of canonical names', () async {
      var records = await fn1cb1ToFuture<List<String>>((host, cb) => new Resolver().resolveCname(host, cb), 'www.belin.io');
      expect(records, orderedEquals(<String>['ns.belin.io']));
    });
  });

  group('.resolveMx()', () {
    test('should resolve the hostname into a list of mail exchange records', () async {
      expect(await fn1cb1ToFuture<List<MXRecord>>((host, cb) => new Resolver().resolveMx(host, cb), 'google.com'), isNotEmpty);
    });
  });

  group('.resolveNs()', () {
    test('should resolve the hostname into a list of name servers', () async {
      var records = await fn1cb1ToFuture<List<String>>((host, cb) => new Resolver().resolveNs(host, cb), 'belin.io');
      expect(records, unorderedEquals(<String>['ns-39-c.gandi.net', 'ns-68-b.gandi.net', 'ns-230-a.gandi.net']));
    });
  });

  group('.resolveSoa()', () {
    test('should resolve the hostname into a start of authority record', () async {
      var record = await fn1cb1ToFuture<SoaRecord>((host, cb) => new Resolver().resolveSoa(host, cb), 'belin.io');
      expect(record.hostmaster, equals('hostmaster.gandi.net'));
      expect(record.nsname, equals('ns1.gandi.net'));
    });
  });

  group('.resolveTxt()', () {
    test('should resolve the hostname into a list of text queries', () async {
      var entries = <String>[];
      var records = await fn1cb1ToFuture<List<List<String>>>((host, cb) => new Resolver().resolveTxt(host, cb), 'belin.io');
      for (var record in records) record.forEach(entries.add);
      expect(entries, unorderedEquals(<String>['firebase=cb-belin', 'google-site-verification=4Lpj0fzF1aePRZ_Zj3SOSn0Gv24ztXPxZ_lkTP1ouFQ']));
    });
  });

  group('.reverse()', () {
    test('should resolve the address into a list of hostnames', () async {
      var hostnames = await fn1cb1ToFuture<List<String>>((addr, cb) => new Resolver().reverse(addr, cb), '192.30.253.113');
      expect(hostnames, orderedEquals(<String>['lb-192-30-253-113-iad.github.com']));
    });
  });
});
