import 'package:nodejs_interop/js.dart';
import 'package:nodejs_interop/node.dart';
import 'package:path/path.dart' as path;
import 'package:test/test.dart';

/// Tests the features of the [RequireFunction] class.
void main() => group('RequireFunction', () {
  group('.cache', () {
    test('should contain at least the main module', () {
      var cache = getProperties(require.cache);
      expect(cache, hasLength(1));
      expect(cache.keys.first, endsWith('require_test.dart.node_test.dart.js'));
    });

    test('should provide info about the cached modules', () {
      var module = getProperties(require.cache).values.first;
      expect(module.id, equals('.'));
      expect(module.loaded, isTrue);
      expect(module.parent, isNull);
    });

    test('should allow the deletion of an entry', () {
      deleteProperty(require.cache, getProperties(require.cache).keys.first);
      expect(getProperties(require.cache), isEmpty);
    });
  });

  group('.main', () {
    test('should be the same value as `process.mainModule`', () {
      expect(require.main, equals(process.mainModule));
      expect(require.main.id, equals('.'));
    });

    test('should not have a parent module', () {
      expect(require.main.parent, isNull);
    });
  });

  group('.call()', () {
    test('should return the `exports` of the required module', () {
      expect(hasProperty(require('console'), 'Console'), isTrue);
      expect(hasProperty(require('dns'), 'Resolver'), isTrue);
    });

    test('should throw an error if the required module does not exist', () {
      expect(() => require('nonexistent'), throwsException);
    });
  });

  group('.resolve()', () {
    test('should return the module identifier for a core module', () {
      expect(require.resolve('child_process'), equals('child_process'));
      expect(require.resolve('dns'), equals('dns'));
    });

    test('should return the absolute path for a local module', () {
      var resolvedPath = require.resolve('del');
      expect(resolvedPath, endsWith(path.join('node_modules', 'del', 'index.js')));
      expect(path.isAbsolute(resolvedPath), isTrue);

      resolvedPath = require.resolve('os-locale');
      expect(resolvedPath, endsWith(path.join('node_modules', 'os-locale', 'index.js')));
      expect(path.isAbsolute(resolvedPath), isTrue);
    });
  });

  group('.resolve.paths()', () {
    test('should return a `null` reference for a core module', () {
      expect(require.resolve.paths('child_process'), isNull);
      expect(require.resolve.paths('dns'), isNull);
    });

    test('should return a list of `node_modules` directories for a local module', () {
      expect(require.resolve.paths('del'), allOf(isNotEmpty, contains(path.normalize('/node_modules'))));
      expect(require.resolve.paths('os-locale'), allOf(isNotEmpty, contains(path.normalize('/node_modules'))));
    });
  });
});
