import 'package:nodejs_interop/js.dart';
import 'package:nodejs_interop/node.dart';
import 'package:nodejs_interop/npm.dart';
import 'package:test/test.dart';

/// Tests the features of the `ipaddr.js` package.
void main() {
  final ipaddr = loadLibrary<IPAddrModule>('ipaddr.js');

  var ipv4Address = '163.172.221.182';
  var ipv4MappedIpv6Address = '::ffff:a3ac:ddb6';
  var ipv6Address = '2a00:1450:4007:811::200e';

  group('IPAddrModule', () {
    group('.fromByteArray()', () {
      test('should return an `IPv4` object for a 4 byte long list', () {
        var address = ipaddr.fromByteArray([0x7f, 0, 0, 1]);
        expect(address.kind(), equals(IPKind.ipv4));
        expect(address.toString(), equals('127.0.0.1'));
      });

      test('should return an `IPv6` object for a 16 byte long list', () {
        var address = ipaddr.fromByteArray([0x20, 1, 0xd, 0xb8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]);
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals('2001:db8::1'));
      });

      test('should throw an error for an invalid byte list', () {
        expect(() => ipaddr.fromByteArray([1, 2, 3]), throwsException);
      });
    });

    group('.isValid()', () {
      test('should return `true` for a valid IPv4 or IPv6 address', () {
        expect(ipaddr.isValid(ipv4Address), isTrue);
        expect(ipaddr.isValid(ipv4MappedIpv6Address), isTrue);
        expect(ipaddr.isValid(ipv6Address), isTrue);
      });

      test('should return `false` for an invalid address', () {
        expect(ipaddr.isValid('foo.bar'), isFalse);
      });
    });

    group('.parse()', () {
      test('should properly parse an IPv4 address', () {
        var address = ipaddr.parse(ipv4Address);
        expect(address.kind(), equals(IPKind.ipv4));
        expect(address.toString(), equals(ipv4Address));
      });

      test('should properly parse an IPv6 address', () {
        var address = ipaddr.parse(ipv6Address);
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals(ipv6Address));
      });

      test('should properly parse an IPv4-Mapped IPv6 address', () {
        var address = ipaddr.parse(ipv4MappedIpv6Address);
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals(ipv4MappedIpv6Address));
      });

      test('should throw an error for an invalid address', () {
        expect(() => ipaddr.parse('foo.bar'), throwsException);
      });
    });

    group('.parseCIDR()', () {
      test('should properly parse an IPv4 address with a CIDR range', () {
        var cidr = ipaddr.parseCIDR('192.168.0.1/32');
        expect(cidr, hasLength(2));
        expect(cidr.first.kind(), equals(IPKind.ipv4));
        expect(cidr.first.toString(), equals('192.168.0.1'));
        expect(cidr.last, equals(32));
      });

      test('should properly parse an IPv6 address with a CIDR range', () {
        var cidr = ipaddr.parseCIDR('2001:db8::/32');
        expect(cidr, hasLength(2));
        expect(cidr.first.kind(), equals(IPKind.ipv6));
        expect(cidr.first.toString(), equals('2001:db8::'));
        expect(cidr.last, equals(32));
      });

      test('should throw an error for an invalid address', () {
        expect(() => IPv4.parse('foo.bar'), throwsException);
      });
    });

    group('.process()', () {
      test('should properly process an IPv4 address', () {
        var address = ipaddr.process(ipv4Address);
        expect(address.kind(), equals(IPKind.ipv4));
        expect(address.toString(), equals(ipv4Address));
      });

      test('should properly process an IPv6 address', () {
        var address = ipaddr.process(ipv6Address);
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals(ipv6Address));
      });

      test('should properly process an IPv4-Mapped IPv6 address', () {
        var address = ipaddr.process(ipv4MappedIpv6Address);
        expect(address.kind(), equals(IPKind.ipv4));
        expect(address.toString(), equals(ipv4Address));
      });

      test('should throw an error for an invalid address', () {
        expect(() => ipaddr.process('foo.bar'), throwsException);
      });
    });

    group('.subnetMatch()', () {
      test('should return the range corresponding to the given IPv4 address', () {
        Object rangeList = jsify({
          'documentationOnly': [ipaddr.parse('192.168.0.1'), 8],
          'tunnelProviders': [[ipaddr.parse('10.100.0.1'), 8], [ipaddr.parse('10.120.0.200'), 8]]
        });

        expect(ipaddr.subnetMatch(ipaddr.parse('192.168.0.127'), rangeList, 'unknown'), equals('documentationOnly'));
      });

      test('should return the range to the given IPv6 address', () {
        Object rangeList = jsify({
          'documentationOnly': [ipaddr.parse('2001:db8::'), 32],
          'tunnelProviders': [[ipaddr.parse('2001:470::'), 32], [ipaddr.parse('2001:5c0::'), 32]]
        });

        expect(ipaddr.subnetMatch(ipaddr.parse('2001:470:8:66::1'), rangeList, 'unknown'), equals('tunnelProviders'));
      });
    });
  });

  group('IPAddress', () {
    group('.kind()', () {
      test('should return "ipv4" for an IPv4 address', () {
        expect(ipaddr.parse(ipv4Address).kind(), equals(IPKind.ipv4));
      });

      test('should return "ipv6" for an IPv6 address', () {
        expect(ipaddr.parse(ipv6Address).kind(), equals(IPKind.ipv6));
        expect(ipaddr.parse(ipv4MappedIpv6Address).kind(), equals(IPKind.ipv6));
      });
    });

    group('.match()', () {
      test('should return `true` if the address falls into a certain CIDR range', () {
        var address = ipaddr.parse('192.168.0.1');
        expect(address.match(ipaddr.parse('192.168.0.25'), 8), isTrue);
        expect(address.match(ipaddr.parseCIDR('192.168.0.25/8')), isTrue);

        address = ipaddr.parse('2001:db8:1234::1');
        expect(address.match(ipaddr.parse('2001:db8::'), 32), isTrue);
        expect(address.match(ipaddr.parseCIDR('2001:db8::/32')), isTrue);
      });

      test('should return `false` if the address does not fall into a certain CIDR range', () {
        var address = ipaddr.parse('192.168.0.1');
        expect(address.match(ipaddr.parse('10.168.0.1'), 8), isFalse);
        expect(address.match(ipaddr.parseCIDR('10.168.0.1/8')), isFalse);

        address = ipaddr.parse('2001:db8:1234::1');
        expect(address.match(ipaddr.parse('2001:db9::'), 32), isFalse);
        expect(address.match(ipaddr.parseCIDR('2001:db9::/32')), isFalse);
      });
    });

    group('.prefixLengthFromSubnetMask()', () {
      test('should return a CIDR prefix length for a valid IPv4 netmask', () {
        expect(ipaddr.parse('255.255.255.240').prefixLengthFromSubnetMask(), equals(28));
      });

      test('should return a `null` reference for a invalid netmask', () {
        expect(ipaddr.parse('255.192.164.0').prefixLengthFromSubnetMask(), isNull);
      });
    });

    group('.range()', () {
      test('should return the special range defined by IPv4 protocol corresponding to the given IPv4 address', () {
        expect(ipaddr.parse('169.254.0.1').range(), IPv4Range.linkLocal);
        expect(ipaddr.parse('127.0.0.1').range(), IPv4Range.loopback);
        expect(ipaddr.parse('224.0.0.1').range(), IPv4Range.multicast);
      });

      test('should return the special range defined by IPv6 protocol corresponding to the given IPv6 address', () {
        expect(ipaddr.parse('fe80::1').range(), IPv6Range.linkLocal);
        expect(ipaddr.parse('::1').range(), IPv6Range.loopback);
        expect(ipaddr.parse('ff01::1').range(), IPv6Range.multicast);
      });
    });

    group('.toByteArray()', () {
      test('should return a 4 byte long list for an IPv4 address', () {
        expect(ipaddr.parse('127.0.0.1').toByteArray(), orderedEquals(<int>[0x7f, 0, 0, 1]));
      });

      test('should return a 16 byte long list for an IPv6 address', () {
        expect(ipaddr.parse('2001:db8::1').toByteArray(), orderedEquals(<int>[0x20, 1, 0xd, 0xb8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]));
      });
    });

    group('.toNormalizedString()', () {
      test('should return the address as-is for an IPv4 address', () {
        expect(ipaddr.parse(ipv4Address).toNormalizedString(), equals(ipv4Address));
      });

      test('should return an expanded address where all zeroes are explicit for an IPv6 address', () {
        expect(ipaddr.parse('2001:0db8::0001').toNormalizedString(), equals('2001:db8:0:0:0:0:0:1'));
      });
    });

    group('.toString()', () {
      test('should return the address as-is for an IPv4 address', () {
        expect(ipaddr.parse(ipv4Address).toString(), equals(ipv4Address));
      });

      test('should return a compact address where all zeroes are implicit for an IPv6 address', () {
        expect(ipaddr.parse('2001:0db8::0001').toString(), equals('2001:db8::1'));
      });
    });
  });

  group('IPv4', () {
    group('.broadcastAddressFromCIDR()', () {
      test('should return the broadcast address for a given IPv4 interface and netmask in CIDR notation', () {
        expect(IPv4.broadcastAddressFromCIDR('172.0.0.1/24').toString(), equals('172.0.0.255'));
      });
    });

    group('.isValid()', () {
      test('should return `true` for a valid IPv4 address', () {
        expect(IPv4.isValid(ipv4Address), isTrue);
        expect(IPv4.isValid('0xc0.168.1.1'), isTrue);
      });

      test('should return `false` for an IPv6 or invalid address', () {
        expect(IPv4.isValid(ipv4MappedIpv6Address), isFalse);
        expect(IPv4.isValid(ipv6Address), isFalse);
        expect(IPv4.isValid('foo.bar'), isFalse);
      });
    });

    group('.isValidFourPartDecimal()', () {
      test('should return `true` for a valid IPv4 address', () {
        expect(IPv4.isValidFourPartDecimal(ipv4Address), isTrue);
      });

      test('should return `false` for an IPv6 or invalid address', () {
        expect(IPv4.isValidFourPartDecimal(ipv4MappedIpv6Address), isFalse);
        expect(IPv4.isValidFourPartDecimal(ipv6Address), isFalse);
        expect(IPv4.isValidFourPartDecimal('0xc0.168.1.1'), isFalse);
        expect(IPv4.isValidFourPartDecimal('foo.bar'), isFalse);
      });
    });

    group('.networkAddressFromCIDR()', () {
      test('should return the network address for a given IPv4 interface and netmask in CIDR notation', () {
        expect(IPv4.networkAddressFromCIDR('172.0.0.1/24').toString(), equals('172.0.0.0'));
      });
    });

    group('.parse()', () {
      test('should properly parse an IPv4 address', () {
        var address = IPv4.parse(ipv4Address);
        expect(address.kind(), equals(IPKind.ipv4));
        expect(address.toString(), equals(ipv4Address));
      });

      test('should throw an error for an IPv6 or invalid address', () {
        expect(() => IPv4.parse(ipv4MappedIpv6Address), throwsException);
        expect(() => IPv4.parse(ipv6Address), throwsException);
        expect(() => IPv4.parse('foo.bar'), throwsException);
      });
    });

    group('.parseCIDR()', () {
      test('should properly parse an IPv4 address with a CIDR range', () {
        var cidr = IPv4.parseCIDR('192.168.0.1/32');
        expect(cidr, hasLength(2));
        expect(cidr.first.kind(), equals(IPKind.ipv4));
        expect(cidr.first.toString(), equals('192.168.0.1'));
        expect(cidr.last, equals(32));
      });

      test('should throw an error for an IPv6 or invalid address', () {
        expect(() => IPv4.parseCIDR('2001:db8::/32'), throwsException);
        expect(() => IPv4.parseCIDR('foo.bar'), throwsException);
      });
    });

    group('.subnetMaskFromPrefixLength()', () {
      test('should return an IPv4 netmask for a valid CIDR prefix length', () {
        expect(IPv4.subnetMaskFromPrefixLength(24).toString(), equals('255.255.255.0'));
        expect(IPv4.subnetMaskFromPrefixLength(29).toString(), equals('255.255.255.248'));
      });
    });

    group('.toIPv4MappedAddress()', () {
      test('should return a corresponding IPv4-mapped IPv6 address', () {
        var address = IPv4.parse(ipv4Address).toIPv4MappedAddress();
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals(ipv4MappedIpv6Address));
      });
    });
  });

  group('IPv6', () {
    group('.isIPv4MappedAddress()', () {
      test('should return `true` for an IPv4-Mapped IPv6 address', () {
        expect(IPv6.parse(ipv4MappedIpv6Address).isIPv4MappedAddress(), isTrue);
      });

      test('should return `false` for an IPv6 address that is not an IPv4-Mapped one', () {
        expect(IPv6.parse(ipv6Address).isIPv4MappedAddress(), isFalse);
      });
    });

    group('.isValid()', () {
      test('should return `true` for a valid IPv6 address', () {
        expect(IPv6.isValid(ipv4MappedIpv6Address), isTrue);
        expect(IPv6.isValid(ipv6Address), isTrue);
      });

      test('should return `false` for an IPv4 or invalid address', () {
        expect(IPv6.isValid(ipv4Address), isFalse);
        expect(IPv6.isValid('foo.bar'), isFalse);
      });
    });

    group('.parse()', () {
      test('should properly parse an IPv6 address', () {
        var address = IPv6.parse(ipv6Address);
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals(ipv6Address));
      });

      test('should properly parse an IPv4-Mapped IPv6 address', () {
        var address = IPv6.parse(ipv4MappedIpv6Address);
        expect(address.kind(), equals(IPKind.ipv6));
        expect(address.toString(), equals(ipv4MappedIpv6Address));
      });
    });

    group('.parseCIDR()', () {
      test('should properly parse an IPv6 address with a CIDR range', () {
        var cidr = IPv6.parseCIDR('2001:db8::/32');
        expect(cidr, hasLength(2));
        expect(cidr.first.kind(), equals(IPKind.ipv6));
        expect(cidr.first.toString(), equals('2001:db8::'));
        expect(cidr.last, equals(32));
      });
    });

    group('.toIPv4Address()', () {
      test('should return an `IPv4` object for an IPv4-Mapped IPv6 address', () {
        var address = IPv6.parse(ipv4MappedIpv6Address).toIPv4Address();
        expect(address.kind(), IPKind.ipv4);
        expect(address.toString(), equals(ipv4Address));
      });

      test('should throw an error for an IPv6 address that is not an IPv4-Mapped one', () {
        expect(() => IPv6.parse(ipv6Address).toIPv4Address(), throwsException);
      });
    });
  });
}
